#! /usr/bin/env python
from __future__ import print_function

import os, sys
import numpy
import argparse

from wurp import get_tpl, TrjWriter
import wurp.parser as parser


"""
Supported parameter and topology format:
    - pdb
    - amber

Supported trajectory format:
    - restart: amber restart
    - ascii: amber trajectory in ascii format
    - netcdf: amber trajectory in netcdf format
"""


# make the process name the CURP.

def do_dryrun(tpl, trj=None, **kwds):
    pass

def gen_trj(tpl, input_trj_fns, input_trj_fmts, trj_type,
            input_fst_lst_int=[(1, -1, 1)], use_pbc=True, **kwds):
    """Generate trajectory iterator"""

    trj_fns_list  = input_trj_fns
    fmts     = input_trj_fmts
    fst_lst_int_list = input_fst_lst_int

    # get number of atoms
    natom = tpl.get_natom()
    use_pbc = True if (tpl.get_pbc_info() and use_pbc) else False

    trj = gen_trajectory_multi(
            trj_fns_list, fmts, natom, fst_lst_int_list, trj_type,
            use_pbc=use_pbc)

    return trj

def gen_trajectory_multi(trj_fns_list, trj_fmts, natom, fst_lst_int_list,
        trj_type='crd',logger=None, use_pbc=False):

    from itertools import izip_longest

    # print(" trj_fns: ", trj_fns_list,
    #       "\n trj_fmts: ", trj_fmts,
    #       "\n fst_lst_int_list: ", fst_lst_int_list)
    for trj_fns, fmt, fst_lst_int in izip_longest(
            trj_fns_list, trj_fmts, fst_lst_int_list):

        # determine parser
        Parser = get_any_parser(fmt, natom, use_pbc, trj_type)

        # generate trajectory
        trj = parser.gen_trajectory(trj_fns, Parser, natom, logger=None,
                use_pbc=use_pbc, first_last_interval=fst_lst_int)

        # iterate
        for istp, snap, box in trj:
            yield istp, snap, box

def get_any_parser(trj_fmt, natom, use_pbc, trj_type='crd'):

    if trj_fmt == 'restart':
        Parser = parser.get_parser(trj_fmt, 'restart')
        Parser.set_trjtype(trj_type)
        return Parser

    else:
        if trj_type == 'crd':
            return parser.get_parser(trj_fmt, 'coordinate')

        elif trj_type == 'vel':
            return parser.get_parser(trj_fmt, 'velocity')

def do_convert_only(tpl, trj, output_trj_fn, output_trj_fmt,
                    trj_type, output_fst_lst_int=[0,-1,1], **kwds):

    # dt = trj.get_dt()
    dt = 0.01
    is_vel = trj_type == 'vel'

    # write trajectory
    writer = TrjWriter(output_trj_fn, output_trj_fmt, dt,
                                   is_vel, output_fst_lst_int)

    for ifrm, trj, box in trj:
        writer.write(ifrm-1, trj, box)
    writer.close()


if __name__ == '__main__':
    from console import arg_conv_trj, exec_command

    parser = arg_conv_trj
    exec_command(parser)



#! /usr/bin/env python2
import sys
import os


def main(ec_fn=None, excluded_list=['WAT'], **kwds):

    if ec_fn is None:
        ec_file = sys.stdin
    else:
        ec_file = open(ec_fn, 'r')
    lines = (line for line in ec_file
             if not line.startswith("#")
             if not line.isspace())

    for line in lines:
        cols = line.split()
        don_line, acc_line, rest = cols[0], cols[1], cols[2:]

        rid1 = int(don_line.split('_')[0])
        rid2 = int(acc_line.split('_')[0])
        rname1 = don_line.split('_')[1]
        rname2 = acc_line.split('_')[1]

        if rid1+1 != rid2: continue
        if rname1 in excluded_list or rname2 in excluded_list: continue

        print(line.strip())


if __name__ == '__main__':
    from wurp.tool.sele.console import exec_command, arg_sel_neighbor
    parser = arg_sel_neighbor()
    exec_command(parser)



#! /usr/bin/env python2
import sys


def main(ec_fn=None, excluded_list=['WAT'], **kwds):

    if ec_fn is None:
        ec_file = sys.stdin
    else:
        ec_file = open(ec_fn, 'r')

    lines = (line for line in ec_file
             if not line.startswith("#")
             if not line.isspace())

    for line in lines:

        cols = line.split()
        don_line, acc_line, rest = cols[0], cols[1], cols[2:]

        rid1 = int(don_line.split('_')[0])
        rid2 = int(acc_line.split('_')[0])

        if rid1+1 == rid2:

            rname1 = don_line.split('_')[1]
            rname2 = acc_line.split('_')[1]

            if rname1 not in excluded_list and rname2 not in excluded_list:
                continue

            # else:
                # print('hoge')
                # print(line)

        print(line.strip())


if __name__ == '__main__':
    from wurp.tool.sele.console import exec_command, arg_sel_noneighbor
    parser = arg_sel_noneighbor()
    exec_command(parser)



#! /usr/bin/env python
from __future__ import print_function

import os, sys
import numpy
import argparse

import wurp

wurp_srcdir = os.path.join(os.environ['CURP_HOME'], 'src')
if wurp_srcdir not in sys.path:
    sys.path.insert(0, wurp_srcdir)
from table.group import gen_residue_group

def main():
    import os, sys

    # parse arguments
    args = get_arguments()

    # get parameters
    cutoff = float(args.cutoff)
    cutoff2 = cutoff*cutoff

    # get topology
    tpl = wurp.get_tpl(args.prmtop_fn)
    natom = tpl.get_natom()

    # generate residue group
    res_info  = tpl.get_residue_info()
    atom_info = tpl.get_atom_info()
    rname_iatoms_pairs = list( (rname, list(numpy.array(iatoms)-1))
            for rname, iatoms in gen_residue_group(
                res_info, atom_info, name_fmt=args.format))

    resnames = [ resname for resname, iatoms in rname_iatoms_pairs ]

    # get trajectory
    crd_parser = wurp.gen_trj(
            args.traj_fns, natom, args.interval, use_pbc=False)
    crds = ( crd for itraj, crd, box in crd_parser )

    # generate residue pair table
    import lib_pickup
    cutoff_method = dict(
            com      = lib_pickup.is_com,
            nearest  = lib_pickup.is_nearest,
            farthest = lib_pickup.is_farthest)[args.cutoff_method]

    import parallel
    par = parallel.ParallelProcessor()

    method = gen_respair_table_par(
            resnames, rname_iatoms_pairs, cutoff_method, cutoff2)
    respair_table_traj = par.run(method, crd=crds)

    if par.is_root():
        respair_iter = gen_respair_table_over_traj(
                respair_table_traj,args.is_union)

        # write residue pair data
        print('# residue pair within {:4.2f}'.format(cutoff))
        npair = 0
        for rname_i, rnames_j in respair_iter:
            if rnames_j:
                npair += len(rnames_j)
                print('[{}]'.format(rname_i))
                print('  '.join(rnames_j))

        print('# number of pairs = ', npair)

    single = False
    if single:

        respair_table_traj = ( gen_respair_table( crd, resnames,
                    rname_iatoms_pairs, cutoff_method, cutoff2)
                    for itraj, crd, box in crd_parser )

        respair_iter = gen_respair_table_over_traj(
                respair_table_traj, args.is_union)


        # write residue pair data
        print('# residue pair within {:4.2f}'.format(cutoff))
        npair = 0
        for rname_i, rnames_j in respair_iter:
            if rnames_j:
                npair += len(rnames_j)
                print('[{}]'.format(rname_i))
                print('  '.join(rnames_j))

        print('# number of pairs = ', npair)


def get_arguments():

    parser = argparse.ArgumentParser(
            ('Pick up the residue pair table to given file as a ndx format.'))

    # add argument definitions
    parser.add_argument(
            dest='traj_fns', nargs='+',
            help='specify the trajectory files.')

    parser.add_argument(
            '-p', '--input-prmtop-file', dest='prmtop_fn', required=True,
            help='specify the prmtop file to be amber format.')
    
    parser.add_argument(
            '-i', '--interval', dest='interval', default=1, type=int,
            help=('specify interval step to perform the calculation for '
                  'trajectory.'))

    parser.add_argument(
            '-m', '--cutoff-method', dest='cutoff_method', default='nearest',
            required=False, choices=['com', 'nearest', 'farthest'],
            help='cutoff method; com, nearest and farthest for residue.')

    parser.add_argument(
            '-c', '--cutoff', dest='cutoff',
            required=False, default=5.0,
            help='specify the cutoff to pick up.')

    parser.add_argument(
            '-t', '--trim-resnames', dest='trim_resnames', nargs='*', 
            required=False, default=[],
            help='residue names for trimming from the target residues.')

    parser.add_argument(
            '-f', '--format', dest='format',
            required=False, default='{rid:05}_{rname}',
            help='specify the format for representing residue identify.')

    parser.add_argument(
            '-U', '--disble-union', dest='is_union',
            required=False, action='store_false',
            help=('whether residue pair table is calculated by union '
                  'or intersection when collecting over trajectoryspecify'))

    # make arguments
    return parser.parse_args()

def cal_dist2(p1, p2):
    # return sum( (p1 - p2)**2 )
    return (p1[0]-p2[0])**2 + (p1[1]-p2[1])**2 + (p1[2]-p2[2])**2

def zip_double(data_iter):

    for i, e1 in enumerate(data_iter):
        for j, e2 in enumerate(data_iter):
            if j <= i: continue
            yield e1, e2

def gen_respair_table(crd, resnames, rname_iatoms_pairs, cutoff_method,cutoff2):

    res_crds = [ crd[iatoms] for rname, iatoms in rname_iatoms_pairs ]

    for i, (rname_i, rcrd_i) in enumerate(zip(resnames, res_crds)):
        resj_cands = []
        for j, (rname_j, rcrd_j) in enumerate(zip(resnames, res_crds)):
            if j <= i: continue

            if cutoff_method(rcrd_i, rcrd_j, cutoff2):
                resj_cands.append(rname_j)

        yield rname_i, resj_cands

def gen_respair_table_par(resnames, rname_iatoms_pairs, cutoff_method, cutoff2):

    def wrapper(crd):
        res_crds = [ crd[iatoms] for rname, iatoms in rname_iatoms_pairs ]
        table = []

        for i, (rname_i, rcrd_i) in enumerate(zip(resnames, res_crds)):
            resj_cands = []
            for j, (rname_j, rcrd_j) in enumerate(zip(resnames, res_crds)):
                if j <= i: continue

                if cutoff_method(rcrd_i, rcrd_j, cutoff2):
                    resj_cands.append(rname_j)

            table += [(rname_i, resj_cands)]

    return wrapper

def is_com(cutoff2):

    def _judge(crd_i, crd_j):
        com_i = numpy.sum(crd_i, 0)
        com_j = numpy.sum(crd_j, 0)
        return cal_dist2(com_i, com_j) <= cutoff2

    return _judge

def is_nearest(cutoff2):

    def _judge(crd_i, crd_j):
        for r_i in crd_i:
            for r_j in crd_j:
                if cal_dist2(r_i, r_j) <= cutoff2:
                    return True
            
        else:
            return False

    return _judge

def is_farthest(cutoff2):

    def _judge(crd_i, crd_j):
        for r_i in crd_i:
            for r_j in crd_j:
                if cal_dist2(r_i, r_j) >= cutoff2:
                    return False
                
        else:
            return True

    return _judge

def gen_respair_table_over_traj(respair_table_traj, is_union=True):

    table = {}
    rnames = []

    for itrj_1, respair_table in enumerate(respair_table_traj):
        print('# itraj = ', itrj_1+1)

        for rname_i, rnames_j in respair_table:

            if rname_i not in table:
                table[rname_i] = set(rnames_j)
                rnames.append( rname_i )
                continue

            if is_union:
                table[rname_i] = table[rname_i] | set(rnames_j)
            else:
                table[rname_i] = table[rname_i] & set(rnames_j)

    for rname_i in rnames:
        yield rname_i, list(table[rname_i])

if __name__ == '__main__':
    main()



from __future__ import print_function

import numpy

import interact_table as it

def parse_target_atoms_line(target_atoms_line, natom):

    atoms = []

    for mask in target_atoms_line:
        if '-' in mask:
            atoms.extend( parse_mask_with_range(mask, atoms, natom) )

        else:
            atoms.append( int(mask) )
            # print('The format of target_atoms keyword is invalid : ', mask)
            # raise 'Error'

    return atoms

def parse_mask_with_range(mask, atoms, natom):

    cols = mask.split('-')

    if cols[0] != '' and cols[1] != '':
        iatm_beg, iatm_end = int(cols[0]), int(cols[1])

    elif cols[0] == '' and cols[1] != '': # -45
        iatm_end = int(cols[1])

        if len(atoms) == 0:
            iatm_beg = 1
        else:
            iatm_beg = atoms[-1] + 1

    elif cols[0] != '' and cols[1] == '': # 45-
        iatm_beg = int(cols[0])
        iatm_end = natom

    else:
        pass

    return range(iatm_beg, iatm_end+1)

def make_iatm_to_itar(target_atoms, natom):
    """Make iatm => itarget dictionary."""
    iatm_to_itar = numpy.zeros( [natom], numpy.int)

    for itar_1, iatm in enumerate(target_atoms):
        iatm_to_itar[iatm-1] = itar_1 + 1

    return iatm_to_itar

def make_interaction_table_current(base_table, target_atoms, natom):
    """Make the interaction table selected by target atoms for current."""

    iatm_to_itar = make_iatm_to_itar(target_atoms, natom)
    iatm_min = min(target_atoms)
    iatm_max = max(target_atoms)

    table = it.InteractionTable(base_table=base_table)

    def squeeze_target_atoms_range(iatm, jatm):
        itar = iatm_to_itar[iatm-1]
        jtar = iatm_to_itar[jatm-1]
        return (itar != 0) or (jtar != 0)

    return table.filter_or(iatm_min, iatm_max).filter_fun(
            squeeze_target_atoms_range)

def make_interaction_table_flux(base_table, target_atoms, natom):
    """Make the interaction table selected by target atoms for flux."""

    iatm_to_itar = make_iatm_to_itar(target_atoms, natom)
    iatm_min = min(target_atoms)
    iatm_max = max(target_atoms)

    table = it.InteractionTable(base_table=base_table)

    def squeeze_target_atoms_range(iatm, jatm):
        itar = iatm_to_itar[iatm-1]
        jtar = iatm_to_itar[jatm-1]
        return (itar != 0) and (jtar != 0)

    return table.filter_and(iatm_min, iatm_max).filter_fun(
            squeeze_target_atoms_range)

if __name__ == '__main__':
    natom = 33
    maskline = ['4',  '7-9', '13-18']
    target_atoms = parse_target_atoms_line(maskline, natom)
    print(target_atoms)

    maskline = ['-15']
    target_atoms = parse_target_atoms_line(maskline, natom)
    print(target_atoms)

    maskline = ['-5', '8-']
    target_atoms = parse_target_atoms_line(maskline, natom)
    print(target_atoms)



"""setting-base - Methods and class of the setting object

Classes:
SettingBase
Section
TypeBase
|+ Int
|+ Float
|+ String
|+ Bool
|+ Choice
|+ List
|+ File
"""
import os
import glob

from exception import WurpException

class SectionNotDefined(WurpException): pass
class InvalidKeyword(WurpException): pass
class InvalidType(WurpException): pass
class InvalidValue(WurpException): pass
class KeywordNotFound(WurpException): pass
class ValueCanNotConvert(WurpException): pass
class FileNotFound(WurpException): pass

################################################################################
class SettingBase:

    def __init__(self, config, check=True):

        # initialize each of sections.
        secnames = [ aname for aname in dir(self) if not aname.startswith('_') ]
        for sname in secnames:
            self[sname].set_name(sname)

        # check the given items in each section and convert to valid items.
        for secname in config.sections():
            try:
                section = getattr(self, secname)
            except AttributeError:
                raise SectionNotDefined(secname)

            pairs = config.items(secname)
            section.set_items(dict(pairs), check)


    def __getitem__(self, attrname):
        return getattr(self, attrname)

    def __str__(self):
        secnames = [ aname for aname in dir(self)
                if not aname.startswith('_') ]
        return "\n\n".join( str(self[sname]) for sname in secnames )

    def __format__(self, fmt_type="rst"):
        if fmt_type == "rst":
            secnames = [ aname for aname in dir(self)
                    if not aname.startswith('_') ]
            return "\n\n".join( format(self[sname], fmt_type)
                    for sname in secnames )
        else:
            return ""


################################################################################
class Section:

    def __init__(self, description='', **kwds):
        self.__desc = description
        self.__key_to_typeobj = kwds
        self.__name = None
        # initialize each of types
        for key, type_obj in self.__key_to_typeobj.items():
            type_obj.set_name(key)

    def __str__(self):
        seclines = "[{sname}]\n\t{desc}\n\n".format(
                sname=self.__name, desc=self.__desc)

        return seclines + "\n\n".join(str(typeobj)
                for typeobj in self.__key_to_typeobj.values())

    def __format__(self, fmt_type="rst"):
        if fmt_type == "rst":
            seclines = "{sname} section\n{secline}\n{desc}\n\n".format(
                    sname=self.__name, secline=(8+len(self.__name))*'~',
                    desc=self.__desc)

            return seclines + "\n\n".join(format(typeobj, fmt_type)
                    for typeobj in self.__key_to_typeobj.values())
        else:
            return ""

        # return '\n\n'.join( ("{sname}\n{secline}\n{desc}\n\n{kwds}"
                # .format(sname=sname, secline=len(sname)*'~',
                    # desc=kwds['description'], kwds=kwds)
                # for sname, kwds in name_to_secstrs.items() ) )


        # for key, type_obj in self.__key_to_typeobj.items():
            # lines.append( '{key}\n {value}'.format(key=key,value=type_obj) )
        # return '\n'.join(lines)

    def set_name(self, name):
        self.__name = name

    def set_items(self, kwds, check=True):
        # check whether keyword is valid or not in the config file.
        for key in kwds:
            self.check_invalid(key)

        # check whether the keyword required in definition exists or not.
        if check:
            for key in self.__key_to_typeobj:
                self.check_require(key, kwds)

        # set value
        for key, value in kwds.items():
            try:
                type_obj = self.__key_to_typeobj[key]
                type_obj.set(value)
            except FileNotFound:
                if check:
                    mes = 'key : {}, value : {}'.format(key, value)
                    raise FileNotFound(mes)
                else:
                    continue

            except InvalidValue:
                mes = 'key : {}, value : {}'.format(key, value)
                raise InvalidValue(mes)
            except ValueCanNotConvert:
                mes = 'key : {}, value : {}'.format(key, value)
                raise ValueCanNotConvert(mes)

    def check_invalid(self, key):
        """Check whether keyword is valid or not in the config file."""
        if key not in self.__key_to_typeobj:
            raise InvalidKeyword(key)

    def check_require(self, key, kwds):
        """Check whether the keyword required in definition exists or not."""
        type_obj = self.__key_to_typeobj[key]
        if type_obj.require():
            if key not in kwds:
                raise KeywordNotFound(key)

    def copy(self, **new_kwds):
        import copy
        old_kwds = copy.deepcopy(self.__key_to_typeobj)
        old_kwds.update(new_kwds)
        return Section(**old_kwds)

    def get_type_items(self):
        return self.__key_to_typeobj

    def get_type(self, attrname):
        self.check()
        return self.__key_to_typeobj[attrname]

    def __getattr__(self, attrname):
        if attrname in self.__key_to_typeobj:
            return self.__key_to_typeobj[attrname].get()
        else:
            raise AttributeError

    def __getitem__(self, attrname):
        if attrname in self.__key_to_typeobj:
            return self.__key_to_typeobj[attrname].get()
        else:
            raise IndexError


################################################################################
from abc import ABCMeta, abstractmethod, abstractproperty
class TypeBase:

    __metaclass__ = ABCMeta

    def __init__(self, default, require, desc='', parser=None):
        self.__desc = desc
        self.__require = require
        self.__parser = parser
        self.__parsed_value = None
        self.__name = None
        self.set_default(default)

    def __eq__(self, type_obj):
        if self.__class__.__name__ != type_obj.__class__.__name__:
            return False
        if self.get() == type_obj.get():
            return True
        else:
            return False

    def __str__(self):
        return "{key} = {value} # {typename}; {desc}".format(
                key=self.__name, value=self._get_value(),
                typename=self.typename, desc=self.__desc)

    def __format__(self, fmt_type="rst"):
        if fmt_type == "rst":
            if self._get_value() is "":
                val = "none"
            elif self._get_value() is True:
                val = "yes"
            elif self._get_value() is False:
                val = "no"
            else:
                val = self._get_value()

            key_val_lines = (
                    "\n**{key} = {value}** (default) : {typename}\n\n"
                    + "    {desc}").format( key=self.__name, value=val,
                            typename=self.typename, desc=self.__desc)

        else:
            key_val_lines = ""

        return key_val_lines

    def set_name(self, name):
        self.__name = name

    def set_default(self, value):
        if not self.check_default(value):
            raise InvalidValue(value)
        self.__value = self.convert_default(value)

    def set(self, value):
        if not self.check(value):
            raise ValueCanNotConvert(value)
        self.__value = self.convert(value)

    def get(self, use_parse=True):
        if use_parse:
            if self.__parser is None:
                return self.__value
            else:
                if self.__parsed_value is None:
                    self.__parsed_value = self.__parser(self.__value)
                return self.__parsed_value
        else:
            return self.__value

    def get_parser(self):
        return self.__parser

    def require(self):
        return self.__require

    def _get_value(self):
        """Get the representable value."""
        return self.get()

    @abstractproperty
    def typename(self):
        return

    @abstractmethod
    def check_default(self, value):
        return

    @abstractmethod
    def convert_default(self, value):
        return

    @classmethod
    @abstractmethod
    def check(cls, raw_value):
        """Check raw value.  This method must use only the class variables."""
        return

    @classmethod
    @abstractmethod
    def convert(cls, raw_value):
        """Return converted value.
        This method must use only the class variables.
        """
        return


################################################################################
class Int(TypeBase):

    def __init__(self, default, require=True, desc='', parser=None):
        TypeBase.__init__(self, default, require, desc, parser)

    @property
    def typename(self): return self.__class__.__name__

    def check_default(self, value):
        return isinstance(value, int)

    def convert_default(self, value):
        return value

    @classmethod
    def check(cls, raw_value):
        try:
            int(raw_value)
            return True
        except ValueError:
            return False

    @classmethod
    def convert(cls, raw_value):
        return int(raw_value)


class Float(TypeBase):

    def __init__(self, default, require=True, desc='', parser=None):
        TypeBase.__init__(self, default, require, desc, parser)

    @property
    def typename(self): return self.__class__.__name__

    def check_default(self, value):
        return isinstance(value, float)

    def convert_default(self, value):
        return value

    @classmethod
    def check(cls, raw_value):
        try:
            float(raw_value)
            return True
        except ValueError:
            return False

    @classmethod
    def convert(cls, raw_value):
        return float(raw_value)


class String(TypeBase):

    def __init__(self, default, require=True, desc='', parser=None):
        TypeBase.__init__(self, default, require, desc, parser)

    @property
    def typename(self): return self.__class__.__name__

    def check_default(self, value):
        return self.check(value)

    def convert_default(self, value):
        return value.strip()

    @classmethod
    def check(cls, raw_value):
        return not raw_value.isspace()

    @classmethod
    def convert(cls, raw_value):
        return raw_value


class Bool(TypeBase):

    _trues  = ['true','True','TRUE','yes','Yes','YES','on','On','ON']
    _falses = ['false','False','FALSE','no','No','NO','off','Off','OFF']

    def __init__(self, default, require=True, desc='', parser=None):
        TypeBase.__init__(self, default, require, desc, parser)

    @property
    def typename(self): return self.__class__.__name__

    def check_default(self, value):
        return isinstance(value, bool)

    def convert_default(self, value):
        return value

    @classmethod
    def check(cls, raw_value):
        if raw_value in cls._trues:  return True
        if raw_value in cls._falses: return True
        return False

    @classmethod
    def convert(cls, raw_value):
        if raw_value in cls._trues:  return True
        if raw_value in cls._falses: return False


class Choice(TypeBase):

    def __init__(self, default, values, value_type, require=True,
            desc='', parser=None):
        self.__Type = value_type

        # store given values
        if len(values) == 0:
            raise ValueCanNotConvert(values)
        self.__values = [ self.__Type( self.__Type.convert(raw_value) )
                          for raw_value in values ]

        TypeBase.__init__(self, default, require, desc, parser)

    @property
    def typename(self):
        choice_str = '|'.join( [v.get() for v in self.__values] )
        return "Choice[{values}]".format(values = choice_str)

    def get(self):
        return TypeBase.get(self).get()

    def check_default(self, value):
        return self.check(value)

    def convert_default(self, value):
        return self.convert(value)

    def check(self, raw_value):
        return self.__Type( self.__Type.convert( raw_value) ) in self.__values
        # if value in [v.get() for v in self.__values]:
        #     return True
        # else:
        #     return False

    def convert(self, raw_value):
        return self.__Type( self.__Type.convert(raw_value) )


class List(TypeBase):

    def __init__(self, default, value_type, require=True, desc='', parser=None):
        self.__Type = value_type
        self.__parsed_value = None
        TypeBase.__init__(self, default, require, desc, parser)

    @property
    def typename(self):
        return "List[{typename}]".format(typename=self.__Type.__name__)

    def get(self):
        values = [value.get() for value in TypeBase.get(self, False)]
        if self.get_parser() is None:
            return values
        else:
            if self.__parsed_value is None:
                self.__parsed_value = self.get_parser()(values)
            return self.__parsed_value

    def check_default(self, value):
        if isinstance(value, list) or isinstance(value, tuple):
            for v in value:
                try:
                    self.__Type(v)
                except InvalidValue:
                    return False
            return True
        else:
            return False

    def convert_default(self, value):
        return [ self.__Type(v) for v in value ]

    def check(self, value):
        for rv in value.split(' '):
            if not self.__Type.check(rv):
                return False
        else:
            return True

    def convert(self, values_str):
        return [ self.__Type( self.__Type.convert(raw_value) )
                 for raw_value in values_str.split(' ') if raw_value != '']

    def _get_value(self):
        return '  '.join(str(v) for v in self.get())


class File(TypeBase):

    def __init__(self, default='', require=True, desc='',
            allow_glob=True, exists=False, parser=None):
        # self._set_defaults(default)
        self.__use_glob = allow_glob
        self.__exists = exists
        TypeBase.__init__(self, default, require, desc, parser)

    @property
    def typename(self):
        return "File"

    def _set_defaults(self, default):
        if default == '':
            self.__filenames = []
        else:
            self.set(default)

    def check_default(self, value):
        if isinstance(value, str):
            return True

        elif isinstance(value, list) or isinstance(value, tuple):
            return True

        else:
            return False

    def convert_default(self, value):
        if isinstance(value, str):
            return [value]
        else:
            files = []
            for path in value:
                if self.__use_glob:
                    for file in sorted(glob.glob(path)):
                        fn = os.path.abspath(file)
                        files.append(fn)
                else:
                    files.append(path)
            return files

    def check(self, value):
        if value == '':
            return False

        for path in value.split(' '):
            if self.__use_glob:
                for fn in sorted(glob.glob(path)):
                    abs_fn = os.path.abspath(fn)
                    if self.__exists:
                        if not os.path.exists(abs_fn):
                            raise FileNotFound(fn)
            else:
                abs_fn = os.path.abspath(path)
                if self.__exists:
                    if not os.path.exists(abs_fn):
                        raise FileNotFound(path)
        return True

    def convert(self, value):
        files = []
        if isinstance(value, str):
            path = value
            if self.__use_glob:
                for fn in sorted(glob.glob(path)):
                    files.append(os.path.abspath(fn))
            else:
                files.append(os.path.abspath(path))

        else:
            for path in value.split(' '):
                if self.__use_glob:
                    for fn in sorted(glob.glob(path)):
                        files.append(os.path.abspath(fn))
                else:
                    files.append(os.path.abspath(path))
        return files

    def _get_value(self):
        return '  '.join(v for v in self.get())


################################################################################
def test_main():

    def parse(xs):
        ys = []
        for x in xs:
            ys.append(x**2)

        return ys

    generic = Section(
        int_test = Int(
            desc = '',
            parser = lambda x: x+10,
            default = 5 ),

        float_test = Float(
            desc = '',
            default = 5.5 ),

        string_test = String(
            desc = '',
            default = 'hoge' ),

        format = Choice(
            desc = '',
            values = ['presto', 'amber'],
            value_type = String,
            default = 'presto' ),

        crd = List(
            desc = '',
            value_type = Int,
            default = [1,2,3] ),

        crd_power = List(
            desc = '',
            value_type = Int,
            parser = parse,
            default = [1,2,3] ),

        target = Choice(
            desc = '',
            values = ['trajectory', 'restart'],
            value_type = String,
            default = 'restart' ),

        bool_test = Bool(
            desc = '',
            default = True),

        file_test = File(exists=False,
            desc = '',
            default = 'flux.dat')

        )

    # print(str(generic))
    # print(generic['format'])

    generic.set_items(dict(float_test=0, file_test='run.ini *g.py'),check=False)
    print(str(generic))


    # print(generic.format)
    # print(generic.int_test)
    # print(generic.float_test)
    # print(generic.bool_test)
    # print(generic.string_test)
    # print(generic.file_test)

if __name__ == '__main__':
    test_main()



"""wurp - Main wurp program.

Developed by Yamato's lab, Nagoya University.
Use the argument -h for a list of available arguments.
Input: .cfg file
(see http://www.comp-biophys.com/resources/wurp-tutorial/tutorials.html
and the tutorial directory for more informations.)
"""

from __future__ import print_function

# Standard module
import sys
import os
import time

import wurp.exception as exception
import wurp.clog as logger
import wurp.setting as st

# Wurp module
SRC_DIR = os.path.dirname(__file__)
sys.path.insert(0, SRC_DIR)

# Write_ functions below are used to print specific messages and data
# in the log.
# Based mostly on logger.py functions, designed for treating the output
# which will appear in the log.


def write_array(array, num_per_line=10):
    """
    Print array (1D tuple, list or numpy array)

    Each line contains num_per_line elements (or less at the end),
    Last character of each element is separated by 5 characters
    from the former element (filled by spaces if needed).

    Parameters
    ----------
    array : array_like
    num_per_line : int, optional
        Number of elements per line

    Examples
    --------
    >>> write_array([0.001, 0.02, 0.1])
    0.001 0.02  0.1
    """

    length = len(array)
    if length//num_per_line != 0:

        for iline_1 in range(length/num_per_line):
            icol_beg, icol_end = 10*iline_1, 10*(iline_1+1)
            line = ' '.join('{:>5}'.format(col)
                            for col in array[icol_beg:icol_end])
            logger.info(line)   # Printed as information, see clog.py

        # Last line
        if length % num_per_line != 0:
            icol_beg = 10 * (length//num_per_line+1)
            line = ' '.join('{:>5}'.format(col)
                            for col in array[icol_beg:])
            logger.info(line)

    else:
        # Last line
        line = ' '.join('{:>5}'.format(col) for col in array)
        logger.info(line)


def write_times(label_time_pairs):
    """Print formatted time with label

    Used to print the time each process took.
    Parameters
    ----------
    label_time_pairs: list of list
        Duration of each process.
        Each element is [label, time].
        label is the process name.
        time is the time the process took.
    """

    logger.info_title('The summary of the elasped times')

    msg = '{:>30} : {:12.5f} [s]'

    for i, (label, dt) in enumerate(label_time_pairs):
        if label == 'End':
            # Total time
            total_time = sum(time for label, time in label_time_pairs)
            logger.info(msg.format('Total wurp time', total_time))
            break
        else:
            logger.info(msg.format(label+' time', dt))


def write_success():
    """Print success message."""
    # Write the citation to logger.
    logger.info_title('CURP finished completely.')


def do_topology(setting):
    """Get topology object.

    Parameters
    ----------
    setting : wurp.setting.Setting
        Topology file and potential container.

    Returns
    -------
    topology : wurp.parser.file_format.topology.TopologyParser
    natom : int
        Number of atom
    """

    import wurp.parser as parser
    file_format = setting.input.format
    fmt_section = getattr(setting, 'input_' + file_format)
    potential = setting.wurp.potential

    # Atom type depends on stress tensor or flux calculation.
    use_atype = setting.wurp.method == 'momentum-current'
    topology = parser.get_tplprm(file_format, fmt_section,
                                 potential, use_atype)
    natom = topology.get_natom()
    return(topology, natom)


def do_target(atom_range, natom):
    """Get target atoms

    Parameters
    ----------
    atom_range : list
        target_atoms field in .cfg file
    topology : wurp.parser.file_format.topology.TopologyParser
    natom : int
        Number of atom

    Returns
    -------
    target_atoms : list of int
        List of targeted atoms indexes.
    """

    from wurp.table import target
    target_atoms = target.parse_target_atoms_line(atom_range,
                                                  natom)

    logger.info_title('Target atoms information')
    logger.info('target atoms list: ' + ', '.join(atom_range))
    logger.info(' ===> ')
    logger.info('show all atoms explicitly:')
    write_array(target_atoms)

    return target_atoms


def gen_tables(inttable):
    """Generate interaction table.

    Parameters
    ----------
    inttable : table.interact_table.InteractionTable

    Returns
    -------
    interact_table : list
    """

    interact_table = list(inttable.gen_divided_table())
    memories = inttable.get_table_memories()

    # Output interact_table.
    logger.info_title('Interaction table informations')
    logger.info('(iatm, jatm_beg, jatm_end)')
    for itab_1, (table, mem) in enumerate(zip(interact_table, memories)):
        msg = '** table {}, memory = {} MB **'
        logger.info(msg.format(itab_1+1, mem/10.0**6))
        for tab in table:
            logger.info(tab)

    return interact_table


def init_current(setting, par):
    """Initialize current calculator and variables.

    Initialize current, heat flux and energy flux calculations.

    Parameters
    ----------
    setting : wurp.setting.Setting
        settings from .cfg configuration file container.
    par : parallel.SequentialProcessor or parallel.ParallelProcessor

    Return
    ------
    cal : wurp.current.flux.EnergyFluxCalculator \
          or wurp.current.current.StressCurrentCalculator
        Made using
         - interaction table between atoms is made in "if do_table",
           taking into account the group_pair table (if do_gpair).
         - group table made in "if do_group"
         - setting, topology and target_atoms.
    writer : wurp.current.writer.MultiFluxWriter or .MultiCurrentWriter
    topology : parser.format.topology.TopologyParser
        Obtained from [input_format] topology_file.
    target_atoms: list of int
        Obtained from [wurp] target_atoms.
        Contains the indices of each atom to be considered.
    label_time_pairs: list of tuples (action, time of action),
        Records time taken by each step.
    """

    label_time_pairs = []

    # Get topology object.
    t_0 = time.time()
    topology, natom = do_topology(setting)
    # Get the list that decomposes all potential.
    decomp_list = topology.get_decomp_list()
    label_time_pairs += [('Topology', time.time()-t_0)]

    # Determine target atoms.
    t_0 = time.time()
    target_atoms = do_target(setting.wurp.target_atoms, natom)
    # Get target atoms names.
    atom_names = topology.get_atom_info()['names']
    target_anames = [str(i_atom) + '_' + atom_names[i_atom-1]
                     for i_atom in target_atoms]
    label_time_pairs += [('Target', time.time()-t_0)]

    # Get atom groups depending on setting.wurp.group_method.
    t_0 = time.time()
    from wurp.table import group

    gname_iatoms_pairs = group.get_group_iatoms_pairs(
        setting, target_atoms,
        topology.get_residue_info(),
        topology.get_atom_info())
    gnames = [gname for gname, iatoms in gname_iatoms_pairs]
    label_time_pairs += [('Group', time.time()-t_0)]

    # Get which group interactions will be considered as group pairs.
    t_0 = time.time()
    if setting.wurp.group_pair_file[0]:
        from wurp.table import group_pair
        gpair_parser = group_pair.GroupPairParser(
            setting.wurp.group_pair_file[0], gnames)
        gpair_table = gpair_parser.get_gpair_table()
    else:
        gpair_table = None
    label_time_pairs += [('Group pair table', time.time()-t_0)]

    # Make interaction tables.
    t_0 = time.time()
    # Make table for nonbonded.
    nonbonded_table = topology.get_nonbonded_table()

    # Make table for target atoms.
    from wurp.table import target
    if setting.wurp.method == 'momentum-current':
        inttable = target.make_interaction_table_current(
            nonbonded_table, target_atoms, natom)
    elif setting.wurp.method in ('energy-flux', 'heat-flux'):
        inttable = target.make_interaction_table_flux(
            nonbonded_table, target_atoms, natom)
    else:
        inttable = nonbonded_table

    # Make table for group pair.
    if gpair_table:
        gp = group_pair.GroupPair(gpair_table, gname_iatoms_pairs, natom)
        inttable = gp.get_inttable_with_gpair(inttable)

    interact_table = gen_tables(inttable)

    label_time_pairs += [('Interaction table', time.time()-t_0)]

    # Decide and prepare a calculator.
    t_0 = time.time()

    import wurp.current as current
    calculator = current.get_calculator(setting)
    cal = calculator()
    cal.prepare(topology=topology, setting=setting,
                target_atoms=target_atoms,
                gname_iatoms_pairs=gname_iatoms_pairs,
                interact_table=interact_table)
    label_time_pairs += [('calculator setting', time.time()-t_0)]

    # Decide Writer.
    t_0 = time.time()

    if par.is_root():
        from wurp.current.writer import get_writer
        writer = get_writer(setting, decomp_list,
                            target_anames, cal.get_groupnames(),
                            gpair_table)
    else:
        writer = None

    label_time_pairs += [('Writer setting', time.time()-t_0)]

    return cal, writer, topology, target_atoms, label_time_pairs


def init_dynamics(setting, par):
    """Initialize microcanonical calculation objects (DEPRECATED).

    Parameters
    ----------
    setting : wurp.setting.Setting
        Settings from .cfg configuration file container.
    par : parallel.SequentialProcessor or parallel.ParallelProcessor

    Output:
    cal : wurp.current.flux.EnergyFluxCalculat
          or wurp.current.current.StressCurrentCalculator
        Made using
         - interaction table between atoms is made in "if do_table",
           taking into account the group_pair table (if do_gpair)x.
         - group table made in "if do_group"
         - setting, topology and target_atoms.
    writer : wurp.current.writer.MultiFluxWriter or .MultiCurrentWriter
    topology : parser.format.topology.TopologyParser
        Obtained from [input_format] topology_file.
    target_atoms: list of int
        Obtained from [wurp] target_atoms.
        Contains the indices of each atom to be considered.
    label_time_pairs: list of tuples (action, time of action),
        Records time taken by each step.
    """
    label_time_pairs = []

    # Get topology object.
    t_0 = time.time()
    topology, natom = do_topology(setting)

    label_time_pairs += [('Topology', time.time()-t_0)]

    # Determine target atoms.
    t_0 = time.time()
    target_atoms = do_target(setting.wurp.target_atoms, natom)
    label_time_pairs += [('Target', time.time()-t_0)]

    # Make table for nonbonded.
    t_0 = time.time()

    inttable = topology.get_nonbonded_table()

    interact_table = gen_tables(inttable)
    label_time_pairs += [('Interaction table', time.time()-t_0)]

    # Defines calculator.
    t_0 = time.time()

    import wurp.dynamics as dynamics
    cal = dynamics.Integrator(topology, setting, interact_table)
    label_time_pairs += [('calculator setting', time.time()-t_0)]

    # Decide writer.
    # In current version the format of the file to write trajectory can be
    # only amber format.
    # We are going to allow the selection of trajectory format.
    t_0 = time.time()

    if par.is_root():
        writer = dynamics.Writer(setting.dynamics)
    else:
        writer = None
    label_time_pairs += [('Writer setting', time.time()-t_0)]

    return cal, writer, target_atoms, topology, label_time_pairs


def get_data_iter(setting, topology, target_atoms):
    """Import targeted atoms trajectories

    Parameters
    ----------
    setting : wurp.setting.Setting
        Settings from .cfg configuration file container.
    topology : parser.format.topology.TopologyParser
        Topology object obtained from [input_format] topology_file.
    target_atoms: list of int
        Indices of each atom to be considered.
        Obtained from [wurp] target_atoms.

    Returns
    -------
    data_iter : generator of tuple
        tuple (cstep_crd, (new_crd, new_vel, pbc_crd))
        cstep_crd : int
            Step of shown coordinates and velocity.
        new_crd : numpy.ma.MaskedArray
            Coordinates of each atom at step cstep_crd.
        new_vel : numpy.ma.MaskedArray
            Velocity of each atom at step cstep_crd.
        pbc_crd : None or tuple of float
            Periodic boundary condition coordinates.
    """

    # Get topology informations.
    natom = topology.get_natom()
    masses = topology.get_atom_info()['masses']
    # Get flag for periodic boundary condition.
    use_pbc = True if topology.get_pbc_info() else False

    # Format
    file_format = setting.input.format
    fmt_section = getattr(setting, 'input_' + file_format)

    # Judge wether the potential used in calculation is valid or not.
    method = setting.wurp.method
    if method == 'momentum-current':
        use_classic = True

    elif method == 'energy-flux':
        use_classic = True

    elif method == 'heat-flux':
        use_classic = True

    elif method == 'electron-transfer':
        use_classic = False

    elif method == 'microcanonical':
        use_classic = True

    else:
        pass

    # Decide trajectory format and parse trajectory files or restart file.
    import wurp.parser as parser

    if use_classic:
        data_iter = parser.gen_phase_trajectory(
            file_format, fmt_section, setting.input.first_last_interval,
            natom, use_pbc, setting.wurp.remove_trans,
            setting.wurp.remove_rotate, target_atoms, masses, logger)

        data_iter.next()
    else:
        data_iter = parser.gen_matrix_ensemble(
            file_format, fmt_section, setting.input.first_last_interval,
            natom, logger)

    return data_iter


def wurp(input_="run.cfg", use_serial=False, vervose=False,
         output_conf_def=False, output_conf_fmtd=False, **kwds):
    """Compute stress tensor or inter-residue energy or heat flow.

    The computation steps are as follow:
    if do_parallel:
        variable par is created.
        par is a SequentialProcessor() or ParallelProcessor() object
        (see parallel.py),
        depending on the command line options and if mpi is run or not.
        clog module is configured, setting the log options.
    if do_write:
        Writes informations like date, license or if wurp is run in
        parallel in log.
    if do_setting:
        setting variable is defined as a setting object (see setting.py),
        storing the configuration file informations.
        From there is either do_current or do_dynamics set as True.
    if do_init:
        init_current() is launched, defining topology and calculator
        variables.
        par_iter() is launched, defining data_iter, a generator going
        through the steps of the trajectory.
        par.run() is launched, defining the results_iter as a generator.
    if do_write:
        results_iter is iterated through, step by step. The calculus
        starts there.
        Each X step the result is printed in the log.
    if do_summary:
        The job is completed. A summary of the time each process took
        and such informations is printed in the log.

    Parameters
    ----------
    input : list of strings
        Configuration files names.
    use_serial : bool, optional
        Whether computations are run in parallel or not (is serial).
    vervose : bool, optional
        If True prints debug level messages in the log.
    output_conf_def :
        If True outputs the config parameters in ini format with default
        values.
    output_conf_fmtd :
        If True outputs the config parameters in rest style with default
        values.
    """

    do_parallel = True
    do_title = True
    do_setting = True
    do_init = True
    do_run = True
    do_write = True
    do_summary = True
    if do_parallel:
        import wurp.parallel as parallel
        if use_serial:
            par = parallel.SequentialProcessor()
        elif parallel.use_mpi:
            par = parallel.ParallelProcessor()
        else:
            par = parallel.SequentialProcessor()

    # Configuration of clog module.
    if vervose:
        logger.log_level = logger.DEBUG
    else:
        logger.log_level = logger.INFO

    logger.print_function = par.write

    if output_conf_def:
        config = st.parse_config()
        setting = st.Setting(config)
        logger.info(setting)
        quit()

    if output_conf_fmtd:
        config = st.parse_config()
        setting = st.Setting(config)
        logger.info(format(setting, "rst"))
        quit()

    # Show Wurp's title.
    if do_title:
        wurp_title = 3*'{:^80}\n'
        logger.info(wurp_title.format(60*'=',
                                      'Wurp Program Start !! ',
                                      60*'-'))
        # Display today's information.
        import datetime
        now = datetime.datetime.today()
        time_fmt = ('TIME STAMP: {year}/{month}/{day:02} '
                    + '{hour:02}:{minute:02}:{second:02}')
        time_stamp = time_fmt.format(year=now.year, month=now.month,
                                     day=now.day, hour=now.hour,
                                     minute=now.minute, second=now.second)
        logger.info('{:>80}'.format(time_stamp))

        # Display the license content.
        logger.info()
        logger.info()
        license_fp = os.path.join(SRC_DIR, 'LICENSE-short.txt')
        with open(license_fp, 'rb') as license_file:
            for line in license_file:
                logger.info(' '*8 + line.strip())
        logger.info()
        logger.info()

        # Display whether the CURP used is serial version or parallel version.
        logger.info_title("Parallel Processing information")
        par_string = 'Use **PARALLEL** caluculation for wurp.'
        ser_string = 'Use **SERIAL** caluculation for wurp.'
        if use_serial:
            logger.info('{:^80}'.format(ser_string))
        else:
            if parallel.use_mpi:
                logger.info('{:^80}'.format(par_string))
            else:
                logger.info('{:^80}'.format(ser_string))

    if do_setting:
        t_0 = time.time()

        # Input file
        config_filename = input_

        # Parse and parse setting
        config = st.parse_config(config_filename)
        setting = st.Setting(config)

        # Set the frequency to output log from the setting parameter.
        logger.set_log_frequency(setting.wurp.log_frequency)

        label_time_pairs = [('Setting', time.time()-t_0)]

    # Decide the calculation method
    do_current = setting.wurp.method in ('momentum-current',
                                         'energy-flux',
                                         'heat-flux')
    do_dynamics = setting.wurp.method == 'microcanonical'

    if do_init and do_current:
        # log
        logger.info("{:^80}".format("**Current** calculation is performed."))

        # Initialize of wurp to get the object for calculation.
        cal, writer, tpl, target_atoms, label_time_pairs_local = \
            init_current(setting, par)
        label_time_pairs += label_time_pairs_local

        t_0 = time.time()
        # Write header of data.
        if par.is_root():
            writer.write_header()
            # Get data iterator. Ex) trajectory, density matrix, ...
            data_iter = get_data_iter(setting, tpl, target_atoms)
        else:
            data_iter = None
        label_time_pairs += [('Data object parse', time.time()-t_0)]

        if do_run:
            t_0 = time.time()

            ############################################################
            results_iter = par.run(cal.run, data=data_iter)
            ############################################################

    elif do_init and do_dynamics:
        logger.info("{:^80}".format("**Dynamics** calculation is performed."))

        # Initialize of wurp to get the object for calculation.
        cal, writer, target_atoms, tpl, label_time_pairs_local \
            = init_dynamics(setting, par)
        label_time_pairs += label_time_pairs_local

        t_0 = time.time()
        if par.is_root():
            # Write header of data.
            writer.write_header()
            # Get data iterator. Ex: trajectory.
            data_iter = get_data_iter(setting, tpl, target_atoms)
        else:
            data_iter = None
        label_time_pairs += [('Data object parse', time.time()-t_0)]

        if do_run:
            t_0 = time.time()

            ############################################################
            results_iter = cal.run(data_iter.next())
            ############################################################

    else:
        logger.info('ERROR: The method given in the CURP is not available.')
        exit()

    if do_write:

        logger.info_title("The each step's calculated informations")
        cur_steps = []
        istep = -1

        dt_writing = 0.0
        if par.is_root():
            t_cal = time.time()
        for istep, (cur_step, results) in enumerate(results_iter):
            t_1 = time.time()
            cur_steps.append(cur_step)
            logger.debug_cycle('    writing the data at step {} ...'
                               .format(cur_step))
            if par.is_root():
                t_write = time.time()
                writer.write(istep, *results)
                logger.debug_cycle('writing at {}: {}'
                                   .format(cur_step, time.time()-t_write))
            t_2 = time.time()
            dt_writing += t_2 - t_1

        if par.is_root():
            dt_cal = time.time() - t_cal

            if istep > -1:
                logger.debug_cycle('total/1process : {} / {}'
                                   .format(dt_cal, dt_cal/float(istep+1)))

    if do_summary:

        # Write the time of writing the data.
        label_time_pairs += [('Run', time.time() - t_0 - dt_writing)]
        label_time_pairs += [('Writing', dt_writing)]
        t_0 = time.time()

        nstep = istep + 1

        if not par.is_root():
            return None

        if istep == -1:
            msg = "The trajectory data wasn't read"
            raise exception.WurpException(msg)

        msg = '{:>20} time : {:12.5f} [s/snapshot]'

        logger.info_title('Detailed timing in calculator object')
        tag_times_pair_iter = cal.gen_time_info()
        total_time = 0.0
        for tag, times in tag_times_pair_iter:
            each_time = sum(times) / float(len(times))
            logger.info(msg.format(tag, each_time))
            total_time += each_time
        logger.info('    ' + 49*'=')
        logger.info(msg.format('Total', total_time))

        # Write timing information of parallel run.
        logger.info_title('Detailed timing for parallel processing')
        msg = '{:>20} time : {:12.5f} [s/snapshot]'
        time_info = par.get_time_info(nstep)
        for label, t in time_info:
            logger.info(msg.format(label, t))

        if par.get_other_time():
            msg = '{:>20} time : {:12.5f} [s/all]'
            logger.info(msg.format('misc', par.get_other_time()))
            logger.info(msg.format('Total MPI', par.get_mpi_time()))

        # Write the calculated trajectory.
        logger.info_title('Steps calculated in trajectory')
        write_array(cur_steps)

        label_time_pairs += [('Post processing', time.time()-t_0)]
        label_time_pairs += [('End', 0.0)]

        # Write the time each process took and success.
        write_times(label_time_pairs)
        write_success()


########################################################################


if __name__ == '__main__':
    from console import arg_compute, exec_command

    parser = arg_compute()
    exec_command(parser)



#! /usr/bin/env python
from __future__ import print_function
import sys
import numpy

from cal_tc import WriterBase, ACFWriter, get_stringnames

import netCDF4 as netcdf

class TCSWriter(WriterBase):
    name = 'tcs'
    unit = '(kcal/mol)^2/fs'

def load_acf(acf_fp, dataname='acf'):

    ncfile  = netcdf.Dataset(acf_fp, mode='r')
    acfs  = ncfile.variables[dataname][:]
    ncfile.close()

    return acfs

def load_acf_first(acf_fp, dataname='acf'):

    ncfile  = netcdf.Dataset(acf_fp, mode='r')

    donors    = get_stringnames( ncfile.variables['donors'][:] )
    acceptors = get_stringnames( ncfile.variables['acceptors'][:] )

    times     = ncfile.variables['time'][:]
    acfs       = ncfile.variables[dataname][:]

    ncfile.close()

    return times, donors, acceptors, acfs

def cal_tcs(acf, dt, coef=1.0):
    """Calculate time series of transport coefficient."""
    return coef * dt*1000.0 * numpy.add.accumulate(acf, 1)

def sum_acf(acf_fps, output_acf_fp, tcs_fp='', coef=1., **kwds):
    import time
    t0 = time.time()

    # first step
    print('loading for {:>5}, '.format(1), end='')
    times, donors, acceptors, acf_sums = load_acf_first(acf_fps[0])
    nacf = len(acf_fps)
    nframe = len(times)
    dt = times[1] - times[0]

    t1 = time.time()
    print('time : {:>7.3f}(s)'.format(t1-t0))
    sys.stdout.flush()
    t0 = t1
    # average acf
    for iacf, acf_fp in enumerate(acf_fps[1:],2):
        print('loading for {:>5}, '.format(iacf), end='')
        acfs = load_acf(acf_fp)
        acf_sums += acfs
        t1 = time.time()
        print('time : {:>7.3f}(s)'.format(t1-t0))
        sys.stdout.flush()
        t0 = t1

    acf_sums = acf_sums/float(nacf)

    # write averaged acf
    writer = ACFWriter(output_acf_fp, nframe, dt)
    writer.write_all(donors, acceptors, acf_sums)

    # write time series transport coefficient
    if tcs_fp:
        tcs_writer = TCSWriter(tcs_fp, nframe, dt)
        tcs = cal_tcs(acf_sums, dt, coef)
        tcs_writer.write_all(donors, acceptors, tcs)


if __name__ == '__main__':
    main()



#! /usr/bin/env python2
from __future__ import print_function
import sys, os

class LabelConverterParser:

    def __init__(self, filename):
        self.parse(filename)

    def parse(self, filename):
        file = open(filename, 'rb')
        label_to_newlabel = {}
        for line in file:
            line = line.strip()
            if not line: continue
            if line.startswith('#'): continue

            key, newlabel = line.split(':')
            label_to_newlabel[key.strip()] = newlabel.strip()

        self.__label_to_newlabel = label_to_newlabel
        file.close()

    def get_label_to_newlabel(self):
        return self.__label_to_newlabel


if __name__ == '__main__':

    ec_file  = sys.stdin
    label_fp = sys.argv[1]

    if not os.path.exists(label_fp):
        for line in ec_file:
            print(line.strip())
        exit()

    label_parser = LabelConverterParser(label_fp)
    label_to_newlabel = label_parser.get_label_to_newlabel()

    for line in ec_file:
        line = line.strip()
        if not line: continue

        if line.startswith('#'):
            print(line.strip())
            continue

        cols = line.split()
        don_line, acc_line, rest = cols[0], cols[1], cols[2:]

        don_line_ = label_to_newlabel.get(don_line)
        if don_line_: don_line = don_line_

        acc_line_ = label_to_newlabel.get(acc_line)
        if acc_line_: acc_line = acc_line_

        print(don_line, acc_line, ' '.join(rest))



from __future__ import print_function
import os, sys
import numpy

topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)
import clog as logger

def cal_com_crd_vel(crd_or_vel, masses):
    """Calculate the coordinate and velocity of the center of mass."""
    
    # com : center of mass
    mass_com = numpy.sum(masses)
    com_crd_vel = numpy.sum(masses[:, None] * crd_or_vel, 0) / mass_com
    return com_crd_vel

    # above code means below code.
    # x_com = numpy.sum(masses * crd_or_vel[:, 0])
    # y_com = numpy.sum(masses * crd_or_vel[:, 1])
    # z_com = numpy.sum(masses * crd_or_vel[:, 2])
    # return numpy.array([x_com, y_com, z_com]) / mass_com

def cal_rotation_omega(crd, vel, masses):
    """Calculate the velocity of rotation coordinate."""

    # calculate the total angular momentum : L[3]
    r_x_mv = masses[:, None] * numpy.cross(crd, vel)
    L = numpy.sum( r_x_mv, 0 )

    # calculate the inertia tensor : I[3,3]
    xs, ys, zs = crd[:, 0], crd[:, 1], crd[:, 2]

    I_xx = numpy.sum(masses * (ys*ys + zs*zs), 0)
    I_yy = numpy.sum(masses * (zs*zs + xs*xs), 0)
    I_zz = numpy.sum(masses * (xs*xs + ys*ys), 0)
    I_xy = - numpy.sum(masses * xs * ys, 0)
    I_xz = - numpy.sum(masses * xs * zs, 0)
    I_yx = I_xy
    I_yz = - numpy.sum(masses * ys * zs, 0)
    I_zx = I_xz
    I_zy = I_yz

    I = numpy.array([
        [I_xx, I_xy, I_xz],
        [I_yx, I_yy, I_yz],
        [I_zx, I_zy, I_zz]
        ])

    # angular momentum: omega = I^-1 L
    I_inv = numpy.linalg.inv(I)
    omegas = numpy.dot(I_inv, L)
    return omegas

def get_crd_vel_trans_rot_removed(masses, crd, vel, target_atoms,
                                rem_trans, rem_rotate):
    target_indices = numpy.array(target_atoms) - 1
    target_masses = numpy.array(masses)[target_indices]

    # get the crd and vel in target atoms
    target_crd = crd[target_indices]
    target_vel = vel[target_indices]

    logger.debug('*** Translation and Rotation ***')
    logger.debug('    calculating within target atoms...\n')
    msg = '{:>30} = ({:8.5f}, {:8.5f}, {:8.5f}) [{:}]'
    # msg = '{:>30} = ({:10.7e}, {:10.7e}, {:10.7e}) [{:}]'

    if rem_trans:
        # get the center of mass for the coordinate
        target_r_com = cal_com_crd_vel(target_crd, target_masses)
        x_com, y_com, z_com = [float(x) for x in target_r_com] # ndarray => float
        logger.info_cycle(msg.format('crd of center of mass', 
                x_com, y_com, z_com, 'A'))

        # get the center of mass for the velocity
        target_v_com = cal_com_crd_vel(target_vel, target_masses)
        vx_com, vy_com, vz_com = [float(v) for v in 1000.0*target_v_com] # ndarray => float
        logger.info_cycle(msg.format('vel of center of mass', 
                vx_com, vy_com, vz_com, 'A/ps'))

        new_crd = crd - target_r_com
        new_vel = vel - target_v_com
        target_crd_minus_com = target_crd - target_r_com
        target_vel_minus_com = target_vel - target_v_com

    else:
        logger.info_cycle(20*' ' + 'remove_trans is off.')
        new_crd = crd
        new_vel = vel
        target_crd_minus_com = target_crd
        target_vel_minus_com = target_vel

    if rem_rotate:

        target_omegas = cal_rotation_omega(
                target_crd_minus_com, target_vel_minus_com, target_masses)

        # log the omega values.
        x_omega, y_omega, z_omega = [float(v) for v in 1000.0*target_omegas]
        logger.info_cycle(msg.format('angular vel for rotation', 
                x_omega, y_omega, z_omega, 'rad/ps'))

        # calculate rotational motion for all of the system using target_omegas.
        target_v_rot = numpy.cross(target_omegas, new_crd)
        new_vel = new_vel - target_v_rot

    else:
        logger.info_cycle(20*' ' + 'remove_rotate is off.')

    logger.info_cycle()

    return new_crd, new_vel

if __name__ == '__main__':

    crd = numpy.array([[-1.2,0.5,-1.3],[0.8,-1.2,1.1]])
    vel = numpy.array([[ 1.0,0.3,0.9],[-0.7,0.9,1.5]])
    mass = numpy.array([1.1, 0.9])

    # cal_rotation_omega(crd, vel, mass)

    new_crd, new_vel = get_crd_vel_trans_rot_removed(
            mass, crd, vel, [1,2], True, True)

    print(new_vel)



from __future__ import print_function

import os, sys
from collections import OrderedDict as odict
import numpy

# wurp mudules
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)
import exception
import clog as logger
import interact_table as it

class InvalidGroupName(exception.WurpException): pass

import ini_parser as ini
class GroupPairParser(ini.IniParser):

    def __init__(self, gpair_fn, gnames):
        ini.IniParser.__init__(self, gpair_fn)
        self.__gnames_base = gnames
        self.parse()
        self.check()

    def check(self):

        gnames = self.get_secnames()

        if not (set(gnames) <= set(self.__gnames_base)):
            invalid_names = list(set(gnames) - set(self.__gnames_base))
            msg = "{} in {} is invalid group names !".format(
                    invalid_names, self.get_filename())
            
            raise InvalidGroupName(msg)

        for gname_i in gnames:
            gnames_j = self[gname_i]

            if not (set(gnames_j) <= set(self.__gnames_base)):
                invalid_names = list(set(gnames_j) - set(self.__gnames_base))
                msg = "{} in {} is invalid group names !".format(
                        invalid_names, self.get_filename())
                raise InvalidGroupName(msg)

    def get_gpair_table(self):
        return list(ini.IniParser.gen_secname_cols_pair(self))


class GroupPair:

    def __init__(self, gpair_table, gname_to_iatoms, natom):
        self.__gpair_table = gpair_table
        self.__gname_to_iatoms = gname_to_iatoms
        self.__natom = natom
        self.__table_with_gpair = None

    def gen_inttable(self):
        """Generate the object equivalent with interaction table
        from group pair table, ex)
        """
        gname_to_iatoms = dict(self.__gname_to_iatoms)

        for gname_i, jgnames in self.__gpair_table:

            jatoms = []
            for gname_j in jgnames:
                jatoms += gname_to_iatoms[gname_j]

            comp_jatoms = list(gen_compressed_iatms(jatoms))
            for iatm in gname_to_iatoms[gname_i]:
                for jatm_beg, jatm_end in comp_jatoms:
                    yield iatm, jatm_beg, jatm_end

    def get_inttable_with_gpair(self, base_table):
        if self.__table_with_gpair is None:
            self.__table_with_gpair = self._make_table_with_gpair(base_table)
        return self.__table_with_gpair

    def _make_table_with_gpair(self, base_table):
        """Make interaction table with group pairs."""

        table_with_gpair = numpy.array(list(self.gen_inttable()))
        base_table = numpy.array(list(base_table))

        import lib_group_pair
        lib_gpair = lib_group_pair.within_gpair
        lib_gpair.setup(table_with_gpair, self.__natom)

        # make interaction table
        new_table, ntable = lib_gpair.get_nonbonded_table(base_table)
        new_table = new_table[:ntable].tolist()
        return it.InteractionTable(base_table=new_table)

def gen_compressed_iatms( iatoms ):
    """Convert: [1,2,3,6,7,8,...,100] => [(1,3),(6,100)]"""

    iatoms.sort()
    iatm_beg = iatoms[0]
    iatm_end = iatoms[0]

    for iatm in iatoms[1:]:

        if iatm != iatm_end+1:
            yield (iatm_beg, iatm_end)
            iatm_beg = iatm

        iatm_end = iatm

    else:
        yield (iatm_beg, iatm_end)


if __name__ == '__main__':
    import os
    gpair_fn = os.path.join(os.environ['CURP_HOME'],
            'test', 'energy-flux', 'pickup', 'gpair.dat')
    tplprm_fn = os.path.join(os.environ['CURP_HOME'],
            'test', 'amber-b2AR', 'stripped.prmtop.gz')

    from parser.amber.topology import TopologyParser, Format2AmberBaseConverter
    raw_tpl = TopologyParser(tplprm_fn)
    raw_tpl.parse()
    tpl = Format2AmberBaseConverter(raw_tpl, use_atomtype=False)
    tpl.convert()

    atom_info = tpl.get_atom_info()
    res_info = tpl.get_residue_info()

    import group
    gnames = [ gname for gname, iatoms
            in group.gen_residue_group(res_info, atom_info) ]

    # for gname in gnames:
        # print(gname)

    gpair_parser = GroupPairParser(gpair_fn, gnames)
    for i in gpair_parser.get_gpair_table():
        print(i)

    # with bm('gen_compressed_iatms'):
        # print()
        # iatoms = [1,2,3,8,9,10,11,12,13]
        # table = gen_compressed_iatms(iatoms)
        # print(list(table))




from __future__ import print_function

import os, sys

import numpy
topdir = os.path.abspath(
        os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)
from exception import WurpException

class MaximumDepthError(WurpException): pass

def gen_searched_wells(rdfs, dr, increment=5, eps=0.0000001):
    for iatm_1, rdf in enumerate(rdfs):
        try:
            r_max,rdf_max,r_min,rdf_min = search_well(rdf, dr, increment, eps)
        except MaximumDepthError as e:
            msg = 'Reached maximum depth: {} at the atom index {}.'
            raise MaximumDepthError(msg.format(e.num_rdfs, iatm_1+1))

        yield r_max, rdf_max, r_min, rdf_min

def search_well(rdf, dr, increment=5, eps=0.0000001):

    # get a index that rdf is maximum.
    num_rdfs = len(rdf)
    rdf_max  = max(rdf)
    i_max = index_float_array(rdf, rdf_max)

    for i in range(num_rdfs):
        i_next = i_max+1 + (i+1)*increment
        rdf_next = rdf[i_max+1:i_next]

        # get minimum and its index
        rdf_min = min(rdf_next)
        i_min = index_float_array(rdf_next, rdf_min)
        i_min += i_max+1

        # judgement
        if num_rdfs == i_min+1:
            break
        else:
            if rdf_min <= rdf[i_min+1]:
            # if rdf_min == 0.0 or rdf_min <= rdf[i_min+1]:
                break

    else:
        error_obj = MaximumDepthError()
        error_obj.num_rdfs = num_rdfs
        raise error_obj

    return dr*(i_max+1), rdf_max, dr*(i_min+1), rdf_min

def index_float_array(array, value, eps=0.0000001):
    criterion = (value-eps <= array) & (array <= value+eps)
    index_item = numpy.where(criterion)
    if len(index_item[0]) == 0:
        return None
    else:
        return index_item[0][0]


if __name__ == '__main__':

    import os, sys
    topdir = os.path.abspath(
            os.path.join(os.path.dirname(__file__), '..', 'parser', 'amber'))
    if topdir not in sys.path:
        sys.path.insert(0, topdir)
    import trajectory
    from calrdf import average_rdf
    
    def print_wells(info):
        rmax, rdf_max, rmin, rdf_min = info
        msg = '{rmax:4.2f} {rdf_max:10.5f}   {rmin:4.2f} {rdf_min:10.5f}'
        print(msg.format(rmax=rmax, rdf_max=rdf_max, rmin=rmin,rdf_min=rdf_min))

    crd_fn = '../parser/amber/test/sam-nwat.mdcrd'
    parser = trajectory.CoordinateParser(crd_fn, 1799)

    rmax = 3.0
    dr   = 0.01

    from benchmarker import Benchmarker
    with Benchmarker(width=30) as bm:

        with bm('calculate average rdf'):
            rdfs = average_rdf(parser, rmax=rmax, dr=dr, interval=1,
                    average=True, per_area=True) 
        # with bm('printing average rdf'):
        #     for iatm_1, rdf in enumerate(rdfs):
        #         print()
        #         print(iatm_1+1)
        #         for rd in rdf:
        #             print(rd)

        with bm('search well of rdf'):
            for iatm_1, tup in enumerate(
                    gen_searched_wells(rdfs, dr, increment=5)):
                print(iatm_1+1)
                print_wells(tup)





"""
For pip install installation.
"""

import os
import fnmatch

from numpy.distutils.command.sdist import sdist
import setuptools

from numpy.distutils.core import setup, numpy_cmdclass
from numpy.distutils.misc_util import Configuration

class sdist_checked(sdist):
    def run(self):
        sdist.run(self)

def ext_modules(config, _dir):
    """Fetch f90 files in src and automatically create an extension"""
    pattern = "*.f90"
    if os.path.isdir(_dir):
        for root, dirs, files in os.walk(_dir):
            match = fnmatch.filter(files, pattern)
            for name in match:
                f90_file = os.path.join(root, name)
                ext_name = os.path.splitext(f90_file)[0].replace("/", ".")
                config.add_extension(ext_name,
                                     [f90_file],
                                     f2py_options=['--quiet']
                                    )

with open('README.rst', 'r') as summary:
    LONG_DESCRIPTION = summary.read()

def run_setup():
    """Setup"""
    config = Configuration(None, '', '')
    ext_modules(config, "wurp")
    config.add_data_files(os.path.join("wurp", "LICENSE-short.txt"))
    setup(
        name="Wurp",
        version="1.2",
        author="Yamato's Lab",
        author_email="yamato@nagoya-u.jp",
        description="Inter-residue Current calculation in Proteins from MD \
            trajectory",
        long_description=LONG_DESCRIPTION,
        long_description_content_type='text/x-rst',
        url=("https://gitlab.com/yamato97/current-calculations-for-proteins"),
        classifiers=[
            "Development Status :: 5 - Production/Stable",
            "Intended Audience :: Science/Research",
            "Operating System :: MacOS",
            "Operating System :: POSIX :: Linux",
            "Programming Language :: Python :: 2",
            "Programming Language :: Python :: 2.7",
            "Programming Language :: Fortran",
            "Topic :: Scientific/Engineering",
            "Topic :: Scientific/Engineering :: Bio-Informatics",
            "Topic :: Scientific/Engineering :: Physics",
            "Topic :: Scientific/Engineering :: Chemistry"
            ],
        cmdclass={"sdist": sdist_checked},

        install_requires=["numpy>=1.11.2,<1.17",
                          "nose",
                          "mpi4py>=2.0",
                          "benchmarker",
                          "pygraphviz<1.6",
                          "netCDF4>=1.2.4"],

        packages=setuptools.find_packages(),

        package_data={'wurp':['volume/random20.pdb.gz']},

        python_requires=">=2.7, <3.0",
        entry_points={
            "console_scripts": [
                "wurp = wurp.console:main",
                ]
            },
        **config.todict()
        )

if __name__ == '__main__':
    run_setup()



from __future__ import print_function

import math
import numpy

def cal_rdf(crd, num_rs, rmax=5.0, dr=0.1, per_area=True):
    """Calclate a radial distribution function on each atom."""
    natom = len(crd)
    rdfs = numpy.zeros((natom, num_rs)) # rdf on each atom.
    
    for rdf_i, r_i in zip(rdfs, crd):
        for r_j in crd:
            r_ij = r_i - r_j
            l_ij = math.sqrt(numpy.dot(r_ij, r_ij))
            if l_ij <= 0.1: continue
            if l_ij >= rmax: continue
            rindex = int(round(l_ij/dr)) - 1
            rdf_i[rindex] += 1

    if per_area:
        for rindex in range(num_rs):
            r = dr * (rindex + 1)
            rdfs[:, rindex] = rdfs[:, rindex] / (r*r)
                
    return rdfs

# replace the calrdf routine to fortran one
import lib_calrdf
cal_rdf = lib_calrdf.calrdf

def average_rdf(parser, rmax=5.0, dr=0.1
        , interval=1, average=True, per_area=True):
    """Calculate a average rdf on each atom."""
    num_rs = int(math.ceil(rmax/dr))
    ntraj = 0

    # calculate first rdfs
    i = 0
    for istep, (crd, box) in parser:
        i += 1
        istep, (crd, box) = parser.next()

        if i == interval:
            ntraj += 1
            break

    natom = len(crd)
    rdfs_total = cal_rdf(crd, num_rs, rmax=rmax, dr=dr, per_area=False)

    # calclate rest rdfs
    i = 0
    for istep, (crd, box) in parser:
        i += 1

        if i == interval:
            ntraj += 1
            rdfs = cal_rdf(crd, num_rs, rmax=rmax, dr=dr, per_area=False)
            rdfs_total += rdfs
            i = 0

    if per_area:
        for rindex in range(num_rs):
            r = dr * (rindex + 1)
            rdfs_total[:, rindex] = rdfs_total[:, rindex] / (r*r)

    if average:
        return rdfs_total/ntraj
    else:
        return rdfs_total

if __name__ == '__main__':
    import os, sys
    topdir = os.path.abspath(
            os.path.join(os.path.dirname(__file__), '..', 'parser', 'amber'))
    if topdir not in sys.path:
        sys.path.insert(0, topdir)
    import trajectory

    crd_fn = '../parser/amber/test/sam-nwat.mdcrd'
    parser = trajectory.CoordinateParser(crd_fn, 1799)

    rmax = 2.5
    dr   = 0.01

    from benchmarker import Benchmarker
    with Benchmarker(width=30) as bm:

        # with bm('python code: calculation'):

        #     rdfs_py = cal_rdf(crd, num_rs, rmax=rmax, dr=dr, per_area=True)

        # with bm('fortran code: calculation'):

        #     rdfs_for = lib_calrdf.calrdf(
        #             crd, num_rs, rmax=rmax, dr=dr, per_area=True)

        # with bm('printing'):
        #     for iatm_1, (rdf_py, rdf_for) in enumerate(zip(rdfs_py, rdfs_for)):
        #         print()
        #         print(iatm_1+1, rdf_py-rdf_for)

        with bm('calculate average rdf'):
            rdfs = average_rdf(parser, rmax=rmax, dr=dr, interval=1,
                    average=False, per_area=False) 
        with bm('printing average rdf'):
            for iatm_1, rdf in enumerate(rdfs):
                print()
                print(iatm_1+1)
                for rd in rdf:
                    print(rd)




from __future__ import print_function

import os, sys
from collections import OrderedDict as odict
import numpy

topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)
import exception
import clog as logger

class SectionNotFoundError(exception.WurpException): pass
class ParserError(exception.WurpException): pass


def get_group_iatoms_pairs(setting, target_atoms, res_info, atom_info):
    """Get group atoms table with reference to setting."""

    method = setting.wurp.group_method

    natom = len( atom_info['elems'] )
    iatm_to_itars = get_iatm_to_itars(target_atoms, natom)

    if method == 'none':
        gname_iatoms_pairs = []

    elif method == 'residue':
        gname_iatoms_pairs = list( gen_group_pairs_with_target(
            gen_residue_group(res_info, atom_info), iatm_to_itars ) )

    elif method == 'united':
        gname_iatoms_pairs = list( gen_group_pairs_with_target(
            gen_united_group(atom_info), iatm_to_itars ) )

    elif method == 'file':
        grp_parser = GroupParser(natom, setting.input.group_file[0])
        gname_iatoms_pairs = list( gen_group_pairs_with_target(
            grp_parser.gen_group_atoms_pairs(), iatm_to_itars ) )

    else:
        raise Exception

    logger.info_title('Group information')
    for gname, iatoms in gname_iatoms_pairs:
        logger.info('[',gname,']')
        logger.info( ' '.join(str(iatm) for iatm in iatoms) )

    return gname_iatoms_pairs


import ini_parser as ini
class GroupParser(ini.IniParser):

    def __init__(self, natom, *args, **kwds):
        self.__natom = natom
        ini.IniParser.__init__(self, *args, **kwds)

    def gen_group_atoms_pairs(self):
        return ini.IniParser.gen_secname_cols_pair(self)

    def _gen_col(self, lines):
        atoms = []
        for line in lines:
            if '=' in line: continue
            for col in line.split():

                if '-' in col:
                    atoms.extend( self._parse_mask_with_range(col, atoms) )

                else:
                    atoms.append( int(col) )

        return atoms

    def _parse_mask_with_range(self, mask, atoms):

        cols = mask.split('-')

        if cols[0] != '' and cols[1] != '':
            iatm_beg, iatm_end = int(cols[0]), int(cols[1])

        elif cols[0] == '' and cols[1] != '': # -45
            iatm_end = int(cols[1])

            if len(atoms) == 0:
                iatm_beg = 1
            else:
                iatm_beg = atoms[-1] + 1

        elif cols[0] != '' and cols[1] == '': # 45-
            iatm_beg = int(cols[0])
            iatm_end = self.__natom

        else:
            pass

        return range(iatm_beg, iatm_end+1)

# for topology and bonded
def squeeze_calculating_bonded(groups, table_name, atoms_table, **other_table):
    """Squeeze the bonded calculation items in crude and fast."""

    # get minimum and maximum
    gmin = min( min(iatoms) for iatoms in groups.values() )
    gmax = max( max(iatoms) for iatoms in groups.values() )

    # make index list
    squeezed_indexes = []
    for index, iatoms in enumerate(atoms_table):
        for iatm in iatoms:
            if gmin <= iatm <= gmax:
                squeezed_indexes.append(index)
                break
            else:
                continue

    # reconstruct data
    results = {}
    results[table_name] = [ atoms_table[index] for index in squeezed_indexes ]
    for key, values in other_table.items():
        results[key] = [ values[index] for index in squeezed_indexes ]

    return results


def squeeze_calculating_bonded2(groups, table_name, atoms_table, **other_table):
    """Squeeze the bonded calculation items in precise and fast."""

    # get flatten group list
    group_atoms = []
    for atoms in groups.values():
        group_atoms.extend(atoms)

    # make index list
    squeezed_indexes = []
    for index, iatoms in enumerate(atoms_table):
        for iatm in iatoms:
            if iatm in group_atoms:
                squeezed_indexes.append(index)
                break
            else:
                continue

    # reconstruct data
    results = {}
    results[table_name] = [ atoms_table[index] for index in squeezed_indexes ]
    for key, values in other_table.items():
        results[key] = [ values[index] for index in squeezed_indexes ]

    return results


def gen_residue_group(res_info, atom_info, name_fmt='', rid_first=1):

    name_fmt = name_fmt if name_fmt else '{rid:05}_{rname}'
    res_ids = res_info['ids']
    res_names = res_info['names']
    natom = len(atom_info['names'])

    rid_last = rid_first

    iatoms = []
    for iatm_1, rid in zip(range(natom), res_ids):

        iatm = iatm_1 + 1

        if rid != rid_last:
            rname = res_names[iatm_1-1]
            gname = name_fmt.format(rname=rname, rid=rid-1)
            yield (gname, iatoms)

            rid_last = rid
            iatoms = []

        iatoms.append( iatm )

    else:
        rname = res_names[-1]
        rid = res_ids[-1]
        gname = name_fmt.format(rname=rname, rid=rid)
        yield (gname, iatoms)
    

def gen_united_group(atom_info, name_fmt=''):

    name_fmt = name_fmt if name_fmt else '{id:05}_{name}'

    # atom_ids = atom_info['ids']
    atom_elems = atom_info['elems']
    atom_names = atom_info['names']
    natom = len(atom_elems)

    # first step
    iatoms = []
    i_beg = 0
    for iatm_1, (elem, name) in enumerate(zip(atom_elems, atom_names)):
        iatm = iatm_1 + 1

        i_beg += 1
        if elem != 'H':
            name_last = name
            iatm_last = iatm
            iatoms.append( iatm )
            break

        iatoms.append( iatm )

    for iatm_1, elem, name in zip(
            range(i_beg, natom+1), atom_elems[i_beg:], atom_names[i_beg:] ):
        iatm = iatm_1 + 1

        if elem != 'H':
            gname = name_fmt.format(name=name_last, id=iatm_last)
            yield (gname, iatoms)
            iatoms = [iatm]
            name_last = name
            iatm_last = iatm

        else:
            iatoms.append( iatm )

    else:
        gname = name_fmt.format(name=name_last, id=iatm_last)
        yield (gname, iatoms)


def gen_group_pairs_with_target(gen_gname_iatoms_pairs, iatm_to_itars):

    for gname, iatoms in gen_gname_iatoms_pairs:

        iatoms_new = []
        for iatm in iatoms:
            if iatm_to_itars[iatm-1] == 0: continue
            iatoms_new.append( iatm )

        if len(iatoms_new) != 0:
            yield (gname, iatoms_new)


def get_iatm_to_itars(target_atoms, natom):
    iatm_to_itars = numpy.zeros( [natom], numpy.int)
    for itar_1, iatm in enumerate(target_atoms):
        iatm_to_itars[iatm-1] = itar_1 + 1
    return iatm_to_itars




#! /usr/bin/env python
from __future__ import print_function

import os, sys
from wurp import TrjWriter

def adjust_vel(tpl, trj, trj_type, output_trj_fn,
               output_trj_fmt, output_fst_lst_int=[(0,-1,1)], **kwds):

    # get velocity trajectory
    is_vel = trj_type=='vel'
    if not is_vel:
        msg = "Adjust velocity command doesn't support coordinate trajectory."
        raise Exception(msg)

    # dt = trj.get_dt()
    dt = 0.01

    # process velocity trajectory
    vel_pair_iter = gen_trj_late(trj, output_fst_lst_int)

    def gen_adjust_vel(vel_pair_iter):
        for trj_prev, trj_now in vel_pair_iter:
            istp_p, vel_p, box_p = trj_prev
            istp,   vel,   box   = trj_now
            # if box_p:
                # box = 0.5*(box+box_p)

            print('adjust-vel: ', istp_p, istp)
            yield istp_p, (0.5*(vel_p + vel), box)

    adjusted_vels = gen_adjust_vel(vel_pair_iter)

    # write trajectory
    writer = TrjWriter(output_trj_fn, output_trj_fmt, dt,
                                   is_vel, (1,-1,1))

    for ifrm, (vel, box) in adjusted_vels:
        writer.write(ifrm-1, vel)
    writer.close()

def gen_trj_late(trj, fst_lst_int=(1,-1,1)):

    first, last, inter = fst_lst_int

    nstep = 0
    first = 0 if first < 0 else first
    last  = 10**18 if last <= -1 else last

    if inter==1:
        for istp_cur, snap, box in trj:
            nstep += 1

            if nstep < first: continue

            if nstep > last:
                yield (istp_prev, snap_prev, box_prev), (istp_cur, snap, box)
                break

            if nstep == first:
                istp_prev = istp_cur
                snap_prev = snap
                box_prev = box
                continue

            yield (istp_prev, snap_prev, box_prev), (istp_cur, snap, box)
            istp_prev = istp_cur
            snap_prev = snap
            box_prev = box

    else:
        cnt_inter = 0
        for istp_cur, snap, box in trj:
            nstep += 1

            if nstep < first: continue
            if nstep > last: break

            cnt_inter += 1
            if nstep == first or cnt_inter == inter:
                cnt_inter = 0
                istp_prev = istp_cur
                snap_prev = snap
                box_prev = box

                # one more step
                nstep += 1
                cnt_inter += 1
                istp_cur, snap, box = trj.next()
                yield (istp_prev, snap_prev, box_prev), (istp_cur, snap, box)

if __name__ == '__main__':

    # test for adjust-vel subcommand
    print('[if interval == 1]')
    vels = ((i-1, 10*i, None) for i in range(1,20+1,1))
    vel_pairs = gen_trj_late(vels, fst_lst_int=(5,-1,1))
    for vel in vel_pairs:
        print(vel)
    print()

    print('[if interval != 1]')
    vels = ((i-1, 10*i, None) for i in range(1,20+1,1))
    vel_pairs = gen_trj_late(vels, fst_lst_int=(2,-1,3))
    for vel in vel_pairs:
        print(vel)
    print()





from .sel_helix import main as helix
from .sel_neighbor import main as neighbor
from .sel_noasa import main as noasa
from .sel_nocap import main as nocap
from .sel_nohelix import main as nohelix
from .sel_noloop import main as noloop
from .sel_noneighbor import main as noneighbor
from .sel_thresh import main as threshold



import calvolume as cv

def get_volume_calculator(topology, setting, target_atoms, gname_iatoms_pairs):
    """Return VolumeCalculator object by setting."""

    # get volume setting
    traj_parser = None
    vsetting = cv.VolumeSetting(topology, setting, traj_parser,
                                target_atoms, gname_iatoms_pairs)

    method = setting.volume.method
    if method == 'none':
        obj = cv.Volume1(vsetting)
    elif method == 'vdw':
        obj = cv.VDWVolumeCalculator(vsetting)
    # elif method == 'smve': # provided, by Srolovitz, Maeda, Vitek, Egami
    #     obj = cv.SMVEVolumeCalculator(vsetting)
    elif method == 'voronoi': # use voronoi polyhedron
        if vsetting.voronoi_solvation == 'none':
            obj = cv.VoronoiVolumeCalculator(vsetting)

        else: # use solvation
            obj = cv.VoronoiVolumeCalculatorWithSolvation(vsetting)

    elif method == 'outer': # use outer volume trajectory already calculated
        obj = cv.OuterVolumeFetcher(vsetting)
    else:
        obj = cv.Volume1(vsetting)

    return obj



#! /usr/bin/env python
from __future__ import print_function

import os, sys
from wurp import TrjWriter


def do_mask(tpl, trj, trj_type, output_trj_fn,
            output_trj_fmt, mask_fn, output_fst_lst_int=[(0,-1,1)], **kwds):

    # dt = trj.get_dt()
    dt = 0.01
    is_vel = trj_type=='vel'
    ids = load_mask(mask_fn)

    # print(ids)

    # apply the mask to the trajectory
    # def gen_mask_trj_and_print(trj):
        # for istp, snap, box in trj:
            # print('istp = ', istp)
    trj_new = ( (istp,snap[ids],box) for (istp,snap,box) in trj )

    # write trajectory
    writer = TrjWriter(output_trj_fn, output_trj_fmt, dt,
            is_vel, output_fst_lst_int)

    for ifrm, trj, box in trj_new:
        writer.write(ifrm-1, trj, box)
    writer.close()

def load_mask(fn):

    with open(fn, 'rb') as file:
        ids_str = (line.split()[1] for line in file
                if line.startswith('ATOM'))

        ids = [ int(id_str)-1 for id_str in ids_str ]

    return ids

if __name__ == '__main__':
    pass



"""Calculate autocorrelation function using flux data
CURP 1.1: Ishikura, 2016. Most of the Layout
CURP 1.2: Yamato, 2019. Heat Flux gestion (and commenting code).

- The type of flux data, i.e., scalar or vector, is specified by
  the newly added argument "-no_axes". With (without) this option,
  scalar (vector) flux is handled, and fortran module lib_acf (lib_hfacf)
  is employed. The lib_hfacf fortran module was newly added.
- For heat-flow, the unit of acf was should be changed to (A*(kcal/mol)/fs)^2.
  (to be worked)
"""

from __future__ import print_function

import sys
import time

import numpy

import netCDF4 as netcdf

# import parallel module
from wurp import ParallelProcessor

# fortran module
from wurp.script.lib_acf import cal_acf
from wurp.script.lib_hfacf import cal_hfacf


def get_stringnames(string_array):
    return [''.join(string.tolist()).strip() for string in string_array]


def get_dt(flux_fn):
    """Get dt between each frame"""

    ncfile = netcdf.Dataset(flux_fn, mode='r')
    times = ncfile.variables['time'][:]
    d_t = times[1] - times[0]
    ncfile.close()

    return d_t


def gen_fluxdata(flux_fn, no_axes):

    t_0 = time.time()

    ncfile = netcdf.Dataset(flux_fn, mode='r')

    icom = 0
    donors = get_stringnames(ncfile.variables['donors'][:])
    acceptors = get_stringnames(ncfile.variables['acceptors'][:])
    npair = len(ncfile.dimensions['npair'])
    t_1 = time.time()
    print('preparing time: {:.3f} [s]'.format(t_1-t_0))

    ncfile.close()

    tsum = 0.0
    for ipair_1, (don, acc) in enumerate(zip(donors, acceptors)):
        print('loading for {}/{} {} {} ... ** '
              .format(ipair_1+1, npair, don, acc), end='')
        sys.stdout.flush()

        t_1 = time.time()
        ncfile = netcdf.Dataset(flux_fn, mode='r')

        if no_axes:
            flux = ncfile.variables['flux'][:, ipair_1, icom]
        else:
            flux = ncfile.variables['flux'][:, :, ipair_1, icom]

        ncfile.close()

        t_2 = time.time()
        print('load time: {:.3f} [s]'.format(t_2-t_1))
        tsum += t_2-t_1

        yield don, acc, flux

    print('total load time: {:.3f} [s]'.format(tsum))


class TCCalculator:
    def __init__(self, frame_range, avg_shift,
                 nsample, coef, no_axes, d_t=0.01):

        self.__fst_lst_intvl = frame_range
        fst, lst, intvl = self.__fst_lst_intvl
        self.nframe_acf = (lst - fst)/intvl + 1
        self.__fst = fst
        self.__lst = lst
        self.__intvl = intvl
        self.__shift = avg_shift
        self.__nsample = nsample
        self.__coef = coef
        self.d_t = d_t
        self.no_axes = no_axes

    def run(self, don, acc, flux):

        if self.no_axes:
            acf = cal_acf(flux, self.nframe_acf,
                                  self.__fst, self.__lst, self.__intvl,
                                  self.__shift, False, self.__nsample)
        else:
            acf = cal_hfacf(flux, self.nframe_acf,
                                      self.__fst, self.__lst, self.__intvl,
                                      self.__shift, False, self.__nsample, 3)

        tc = self.cal_tc(acf)
        return tc, acf

    def run_mpi(self, data, **other):
        t_0 = time.time()
        don, acc, flux = data

        if self.no_axes:
            acf = cal_acf(flux, self.nframe_acf,
                                  self.__fst, self.__lst, self.__intvl,
                                  self.__shift, False, self.__nsample)
        else:
            acf = cal_hfacf(flux, self.nframe_acf,
                                      self.__fst, self.__lst, self.__intvl,
                                      self.__shift, False, self.__nsample, 3)

        tc = self.cal_tc(acf)

        # self.print('cal time:', time.time()-t_0)
        print('    cal time: {:.3f} [s] for {} {}'
              .format(time.time()-t_0, don, acc))
        return don, acc, tc, acf

    def run_all(self, flux_iter):
        fst, lst, intvl = self.__fst_lst_intvl
        for flux in flux_iter:
            t_0 = time.time()

            if self.no_axes:
                acf = cal_acf(flux, self.nframe_acf, fst, lst, intvl,
                                      self.__shift, False, self.__nsample)
            else:
                acf = cal_hfacf(flux, self.nframe_acf, fst, lst,
                                          intvl, self.__shift, False,
                                          self.__nsample, 3)

            tc = cal_tc(acf, self.d_t, self.__coef)

            t_1 = time.time()
            print('    cal time: {:.3f} [s]'.format(t_1-t_0))

            yield tc, acf

    def cal_tc(self, acf):
        """Calculate final transport  coefficient."""
        return self.__coef * self.d_t*1000.0 * numpy.sum(acf)

    def get_times(self):
        return numpy.arange(0.0, self.nframe_acf*self.d_t, self.d_t)

    def print(self, *args, **kwds):
        print(*args, **kwds)


class TCWriter:

    """
    Write transport coefficient.
    """

    def __init__(self, fn):
        self.__fn = fn
        self.__file = open(self.__fn, 'wb')

    def write(self, don, acc, tc):
        fd = self.open()
        fd.write('{:>12} {:>12}  {}\n'.format(don, acc, tc))
        fd.flush()

    def open(self):
        if self.__file is None:
            fd = open(self.__fn, 'rb+')
            self.__file = fd

        return self.__file

    def close(self):
        if self.__file:
            self.__file.close()
            self.__file = None


class WriterBase:
    version = 0.1
    name = ''
    unit = ''

    def __init__(self, fp, nframe, d_t, label='test', title=''):

        self.__fp = fp
        self.__ncfile = None
        self.__nframe = nframe
        self.__title = title
        self.__label = label
        self.__d_t = d_t

        self.setup()

    def setup(self):

        ncfile = netcdf.Dataset(self.__fp, clobber=True,
                                mode='w', format='NETCDF3_64BIT')

        # set global attributes
        ncfile.title = self.__title
        ncfile.application = 'the CURP program'
        ncfile.program = 'cal-tc'
        ncfile.programVersion = str(0.7)
        ncfile.Convetsions = 'CURP'
        ncfile.ConvetsionVersion = str(self.version)

        # create dimensions
        ncfile.createDimension('npair', None)
        ncfile.createDimension('nframe', self.__nframe)
        ncfile.createDimension('nchar', 20)

        # create variables
        # time
        nc_time = ncfile.createVariable('time', 'f4', ('nframe',))
        nc_time.units = 'picosecond'
        d_t = self.__d_t
        times = [d_t * ifrm_1 for ifrm_1 in range(self.__nframe)]
        nc_time[:] = times[:]

        # donor and acceptor
        ncfile.createVariable('donors', 'c', ('npair', 'nchar'))
        ncfile.createVariable('acceptors', 'c', ('npair', 'nchar'))

        # trajectory
        nc_data = ncfile.createVariable(self.name, 'f4', ('npair', 'nframe'),)
        nc_data.units = self.unit

        ncfile.sync()
        ncfile.close()

    def write(self, ipair_1, don, acc, data):

        ncfile = self.open()

        nc_don = ncfile.variables['donors']
        nc_acc = ncfile.variables['acceptors']
        nc_data = ncfile.variables[self.name]

        nc_don[ipair_1] = list(don.ljust(20))
        nc_acc[ipair_1] = list(acc.ljust(20))
        nc_data[ipair_1] = data.ravel()

        self.close()

    def write_all(self, donors, acceptors, datas):

        ncfile = self.open()

        nc_don = ncfile.variables['donors']
        nc_acc = ncfile.variables['acceptors']
        nc_data = ncfile.variables[self.name]

        for ipair_1, (don, acc) in enumerate(zip(donors, acceptors)):
            nc_don[ipair_1] = list(don.ljust(20))
            nc_acc[ipair_1] = list(acc.ljust(20))

        nc_data[:] = datas

        self.close()

    def open(self):
        if self.__ncfile is None:
            ncfile = netcdf.Dataset(self.__fp, mode='r+')
            self.__ncfile = ncfile

        return self.__ncfile

    def close(self):
        if self.__ncfile:
            self.__ncfile.sync()
            self.__ncfile.close()
            self.__ncfile = None


class ACFWriter(WriterBase):
    name = 'acf'
    unit = '(kcal/mol/fs)^2'


def cal_tc(flux_fn, tc_fn="", acf_fn="", acf_fmt="netcdf",
           frame_range=[1, -1, 1], avg_shift=1, nsample=0, d_t=None,
           coef=1.0, use_debug=False, no_axes=False, **kwds):

    ti = time.time()
    par = ParallelProcessor()

    # determine delta t

    if d_t is None:
        d_t = get_dt(flux_fn)
        d_t = d_t * frame_range[2]    # in ps
    else:
        d_t = d_t * frame_range[2]    # in ps

    # prepare calculator
    cal = TCCalculator(frame_range, avg_shift, nsample, coef, no_axes, d_t)
    cal.print = par.write

    # prepare writer
    if par.is_root():
        tc_writer = TCWriter(tc_fn)

    # create ACF writer if it is necessary.
    if acf_fn:
        if par.is_root():
            acf_writer = ACFWriter(acf_fn, cal.nframe_acf, d_t)
    else:
        acf_writer = None

    # prepare flux data
    if par.is_root():
        fluxdata_iter = gen_fluxdata(flux_fn, no_axes)
    else:
        fluxdata_iter = None

    results_iter = par.run(cal.run_mpi, data=fluxdata_iter)

    for ipair_1, (don, acc, tc, acf) in enumerate(results_iter):
        if par.is_root():
            tc_writer.write(don, acc, tc)

            if acf_writer:
                t_acf = time.time()
                acf_writer.write(ipair_1, don, acc, acf)
                print('    writing acf: {:.4f} [s]'.format(time.time()-t_acf))

    if par.is_root():
        tc_writer.close()

    if par.is_root():
        print('total time: {:.3f} [s]'.format(time.time()-ti))

        print('==========================')
        print('    finished completely   ')
        print('==========================')



if __name__ == '__main__':
    from console import arg_cal_tc, exec_command

    parser = arg_cal_tc()
    exec_command(parser)



import os, sys

class Atom:
    terminal = False

    def __str__(self):
        return '<Atom, id={}, name={}, rid={}, rname={}>'.format(
                self.id, self.name, self.rid, self.rname)

class System:

    def __init__(self, pdb_fpfd):

        atoms = self._gen_pdbatom(pdb_fpfd)
        self.__atoms = list(atoms)
        self.__natom = len(self.__atoms)
        self.__rid_to_atoms = self._make_rid_to_atoms()

        self.__cur_iatm = 0

    def __next__(self):
        self.__cur_iatm += 1
        if self.__cur_iatm <= self.__natom:
            return self.__atoms[self.__cur_iatm-1]
        else:
            raise StopIteration
    next = __next__
    
    def __iter__(self):
        return self

    def atom(self, iatm):
        return self.__atoms[iatm-1]

    def to_string(self):
        return "\n".join(self.gen_pdbline())

    def residue(self, resid):
        return self.__rid_to_atoms.get(resid)

    def gen_pdbline(self):
        """Generate the pdb string from atom list."""

        line_template = (
            'ATOM  {id:>5} {name:<4} {rname:>3} {cname:>1}{rid:>5}'
            '   {x:8.3f}{y:8.3f}{z:8.3f}{occ:6.2f}{bfact:6.2f}'
            '          {elem:<2}  '
        )

        for iatm_1, atom in enumerate(self.__atoms):
            x, y, z = atom.pos
            yield line_template.format(
                id=iatm_1+1, name=atom.name, rname=atom.rname, cname=atom.chain,
                rid=atom.rid, x=x, y=y, z=z,
                occ=atom.occupancy, bfact=atom.bfactor, elem=atom.element)

            if atom.terminal:
                yield 'TER'

    def _gen_pdbatom(self, pdb_fpfd, vervose=False):
        """Generate atoms from pdb file."""

        rid_shift = 0

        if type(pdb_fpfd) == file:
            pdb_file = pdb_fpfd

        else:
            pdb_fpfd = os.path.expanduser(pdb_fpfd)
            if pdb_fpfd.endswith('.gz'):
                import gzip
                pdb_file = gzip.open(pdb_fpfd, 'r')
            else:
                pdb_file = open(pdb_fpfd, 'r')

        atom_prev = None

        # get first atom
        for line in pdb_file:
            if line[0:6].strip() in ['ATOM', 'HETATM']:
                atom_prev = parse_pdbline(line)
                natm = 1
                break
            else:
                continue

        # get all atoms
        for line in pdb_file:

            if line[0:6].strip() == 'TER':
                atom_prev.terminal = True

            if line[0:6].strip() not in ['ATOM', 'HETATM']: continue

            atom = parse_pdbline(line)
            natm += 1

            yield atom_prev

            # correction of residue id for pdb file obtained by Amber tools.
            if atom_prev.rid == 9999 and atom.rid == 0:
                rid_shift += 10000

            atom.rid += rid_shift
            
            atom_prev = atom

        else:
            atom_prev.terminal = True
            yield atom_prev

        if vervose:
            print('REMARK the number of atoms that was read: {}'.format(natm))

        if pdb_file.name != '<stdin>':
            pdb_file.close()
        
    def _make_rid_to_atoms(self):

        rid_to_atoms = {}
        prev_rid = 1
        atoms = []

        for atom in self.__atoms:

            if atom.rid != prev_rid:
                rid_to_atoms[prev_rid] = atoms
                atoms = []

            atoms.append(atom)
            prev_rid = atom.rid

        else:
            rid_to_atoms[prev_rid] = atoms

        return rid_to_atoms

def parse_pdbline(line):
    atom = Atom()

    try:

        atom.id = int(line[6:11])
        atom.name = line[12:16].strip()
        atom.rname = line[17:20].strip()
        atom.chain = line[21:22].strip()
        atom.rid = int(line[22:28])
        x = float(line[30:38])
        y = float(line[38:46])
        z = float(line[46:54])
        atom.pos = x, y, z

        occ = line[54:60].strip()
        atom.occupancy = float(occ) if occ else 1.00

        bfact =  line[60:66].strip()
        atom.bfactor = float(bfact) if bfact else 0.00
    except:
        print(line)
        raise

    # get elements
    if len(atom.name) == 1:
        atom.element = atom.name[0]
    else:
        atom.element = atom.name[1] if atom.name[0].isdigit() else atom.name[0]

    return atom

def gen_pdbatom(pdb_fn, vervose=False):
    """Generate atoms from pdb file."""

    if pdb_fn.endswith('.gz'):
        import gzip
        pdb_file = gzip.open(pdb_fn, 'rb')
    else:
        pdb_file = open(pdb_fn, 'r')

    atom_prev = None

    # get first atom
    for line in pdb_file:
        if line[0:6].strip() in ['ATOM', 'HETATM']:
            atom_prev = parse_pdbline(line)
            natm = 1
            break
        else:
            continue

    # get all atoms
    for line in pdb_file:

        if line[0:6].strip() == 'TER':
            atom_prev.terminal = True

        if line[0:6].strip() not in ['ATOM', 'HETATM']: continue

        atom = parse_pdbline(line)
        natm += 1

        yield atom_prev
        
        atom_prev = atom

    else:
        atom_prev.terminal = True
        yield atom_prev

    if vervose:
        print('REMARK the number of atoms that was read: {}'.format(natm))

    pdb_file.close()

def getPDB(pdb_fn):
    """ get molecules data from pdb file
    """

    pdb_file = open(pdb_fn, 'r')

    system = []
    for line in pdb_file:
        if line[0:6].strip() != "ATOM": continue
        atom = {}
        atom["id"] = int(line[6:11])
        atom["name"] = line[12:16].strip()
        atom["rname"] = line[17:20].strip()
        atom["rid"] = int(line[22:26])
        x = float(line[30:38])
        y = float(line[38:46])
        z = float(line[46:54])
        atom["xyz"] = [x, y, z]

        system.append(atom)

    pdb_file.close()
    return system


def gen_pdbline(atoms):
    """Generate the pdb string from atom list."""

    line_template = (
            '{record:<6}{id:>5} {name:<4} {rname:>3} {cname:>1}{rid:>5}'
            '   {x:8.3f}{y:8.3f}{z:8.3f}{occ:6.2f}{bfact:6.2f}'
            '          {elem:<2}  '
    )

    for iatm, atom in enumerate(atoms,1):
        x, y, z = atom.pos
        aid = atom.id if hasattr(atom, 'id') else iatm
        yield line_template.format(record='ATOM',
            id=aid, name=atom.name, rname=atom.rname, cname=atom.chain,
            rid=atom.rid%10000, x=x, y=y, z=z,
            occ=atom.occupancy, bfact=atom.bfactor, elem=atom.element)

        if atom.terminal:
            yield 'TER'

def gen_residues(atoms):

    try:
        atom = atoms.next()
    except AttributeError:
        atom  = atoms[0]
        atoms = atoms[1:]

    res_atoms = [atom]
    res_id = atom.rid

    for atom in atoms:
        if res_id != atom.rid:
            yield res_atoms
            res_atoms = []

        res_atoms.append(atom)
        res_id = atom.rid

    yield res_atoms


if __name__ == '__main__':

    fn = '~/Desktop/complex.pdb'
    pdb_fn = os.path.expanduser(fn)

    atoms = gen_pdbatom(pdb_fn)
    for line in gen_pdbline(atoms):
        print(line)

    pdb_fn = './testdata/system.pdb'
    system = System(pdb_fn)
    # print(system.to_string())
    
    # iatm = 5
    # print(system.atom(iatm))
    for atom in system.residue(5):
        print(atom)

    for atom in system.residue(6):
        print(atom)

    for atom in system.residue(1500):
        print(atom)




#! /usr/bin/env python2
import sys
import os


def main(ec_fn, threshold, **kwds):
    # remove the only space line and the comment line
    if ec_fn is None:
        ec_file = sys.stdin
    else:
        ec_file = open(ec_fn, 'r')
    line_iter = (line.strip() for line in ec_file
                 if not line.isspace())
    ec_file.close()

    # store
    rkey_to_count = {}
    for line in line_iter:
        if line.startswith('#'):
            print(line)
            continue

        ec = float(line.split()[-1])
        if ec >= threshold: print(line)


if __name__ == '__main__':
    from wurp.tool.sele.console import exec_command, arg_sel_thresh
    parser = arg_sel_thresh()
    exec_command(parser)
